/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Writer;

import org.jboss.logging.Logger;

/**
 * AbstractResponse provides a base implementation of the Response interface.
 * <p>
 * This class should be extended to provide concrete implementations for each
 * protocol.
 *
 * @author Eric Daugherty
 * @author Andrew C. Oliver (acoliver@jboss.org)
 */
public class AbstractResponse implements Response {

    protected Request request;
    protected OutputStream out;
    protected Protocol protocol;
    protected boolean finish;
   

    public Request getRequest() {
        return request;
    }

    public OutputStream getOutputStream() throws IOException {
        return out;
    }

    public PrintWriter getWriter() throws IOException {
        return new WrappedWriter(new PrintWriter(out));
    }

    public Protocol getProtocol() {
        return protocol;
    }

    public boolean isFinish() {
        return finish;
    }

    /**
     * used to get a writer if we can't (yet) get the log.  Basically used only by greet() should stay that way.
     * @param print writer to wrap
     * @return WrappedWriter 
     */
    public static PrintWriter getWrappedWriter(PrintWriter p) {
        return new WrappedWriter(p);
    }

}

/**
 * used to log the server responses to the conversational log (not log4j presently)
 * @author Andrew C. Oliver <acoliver et jboss det org>
 */
class WrappedWriter extends PrintWriter {
    /**
     * this will actually do the writing, we just wrap it (hence the name ;-) )
     */
    private Writer writer;
    /**
     * log we're writing to, should be the serverLog 
     */
    private Logger log = Logger.getLogger(WrappedWriter.class);
    /**
     * SMTP dictates this EOL sequence
     */
    protected String lineSeparator = "\r\n";

    /**
     * constructor for wrapped writer which sets up the printwriter to use and the serverLog to use
     * @param writer to wrap
     * @param log to log the responses to
     */
    public WrappedWriter(Writer writer) {
        super(writer,false);
        this.writer = writer;
    
    }
    
    public void println(String arg0) {
        //System.out.println("SERVER OUTPUT: "+arg0);
        //System.out.print(HexDump.toHex((arg0 + lineSeperator).getBytes()));
        super.write(arg0);  // some clients barf if this isn't the line sep
        super.write(lineSeparator);
        if (log.isTraceEnabled()) {
            log.trace(arg0);
        }
    }

    public void flush() {
      //  System.out.println(this.out.c)
        super.flush();
    }

}
