/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;

import org.apache.log4j.Logger;
import org.jboss.mail.util.JBMailJMXUtil;


/**
 *
 * @author Andrew C. Oliver <acoliver ot jboss dat org>
 */
public class ConnectionHandler implements Runnable {
	
	final static Logger log = Logger.getLogger(ConnectionHandler.class);
	
    private Socket socket;
    private String protocolName;
    private long timeout;
    private long life;
    private Protocol protocol;
    private Reaper reaper;

	private ServerSocketChannel ssc;

	private SocketChannel ioChannel;
    
    public ConnectionHandler(Socket socket, String protocolName, long timeout, long life, Reaper reap) {
        this.socket = socket;
        this.protocolName = protocolName;
        this.timeout = timeout;
        this.life = life;
        this.reaper = reap;
    }
    
    
    /**
	 * @param ssc
	 * @param protocolName2
	 * @param timeout2
	 * @param life2
	 * @param reaper2
     * @throws IOException
	 */
	public ConnectionHandler(ServerSocketChannel ssc, String protocolName, long timeout, long life, Reaper reap) throws IOException {
		this((Socket)null,protocolName,timeout,life,reap);
		this.ssc = ssc;
		this.ioChannel = ssc.accept();
	}


	/* (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {
    		if(ssc == null && socket != null) {
    			runSocket();
    		} else {
    			runChannel();
    		}
    }


	/**
	 * 
	 */
	private void runChannel() {
		boolean loop = true;
        Thread thisthread = Thread.currentThread();
        reaper.register(ssc,thisthread,timeout, life);
        try {
            
            try {
                this.protocol = (Protocol) JBMailJMXUtil.getService(protocolName);
            } catch (Exception e1) {
                throw new RuntimeException(e1.toString()+" cannot get protocol "+protocolName,e1);
            }
            protocol.setState(ServerThread.STATE_CLIENT_ADDRESS,ssc.socket().getInetAddress());
            protocol.setState(ServerThread.STATE_LOCAL_ADDRESS,ssc.socket().getLocalSocketAddress());
            Selector selector = Selector.open();
		   ioChannel.configureBlocking(false);
		   ioChannel.register(selector, SelectionKey.OP_READ|SelectionKey.OP_WRITE);
		   
            protocol.greet(ioChannel);
          
         /*   while (loop && input.read() != -1 && life > System.currentTimeMillis()) {
                input.reset();
                // create Request object and parse
                Request request = protocol.parseRequest(input, socket);
                reaper.lock(thisthread);
                reaper.update(thisthread);
                input.mark(2); //we've got our command...mark our new spot

                // create Response object
                Response response = protocol.handleRequest(output, request);
                reaper.unlock(thisthread);
                Thread.sleep(50); //so we can be interrupted
                input.mark(2);
                loop = !response.isFinish();
            }*/
        } catch (IOException e) {
            throw new RuntimeException(e);
		} //catch (InterruptedException e2) {
		//	e2.printStackTrace();
		//} 
        finally{
        	//Close the socket
			try {
				this.socket.close();
			} catch (IOException e1) {
				e1.printStackTrace(); 
			}
			log.debug("ConnectionHandler finished");
        }
	}


	/**
	 * 
	 */
	private void runSocket() {
        InputStream input;
        Thread thisthread = Thread.currentThread();
        reaper.register(socket,thisthread,timeout, life);
        try {
            
            try {
                this.protocol = (Protocol) JBMailJMXUtil.getService(protocolName);
            } catch (Exception e1) {
                throw new RuntimeException(e1.toString()+" cannot get protocol "+protocolName,e1);
            }
            protocol.setState(ServerThread.STATE_CLIENT_ADDRESS,socket.getInetAddress());
            protocol.setState(ServerThread.STATE_LOCAL_ADDRESS,socket.getLocalAddress());
      
            input = new BufferedInputStream(socket.getInputStream());
     
            OutputStream output = socket.getOutputStream();
            protocol.greet(output);
      //  ((AbstractProtocol)protocol).setServerThread(this);
            boolean loop = true;
            input.mark(2);            
            while (loop && input.read() != -1 && life > System.currentTimeMillis()) {
                input.reset();
                // create Request object and parse
                Request request = protocol.parseRequest(input, socket);
                reaper.lock(thisthread);
                reaper.update(thisthread);
                input.mark(2); //we've got our command...mark our new spot

                // create Response object
                Response response = protocol.handleRequest(output, request);
                reaper.unlock(thisthread);
                Thread.sleep(50); //so we can be interrupted
                input.mark(2);
                loop = !response.isFinish();
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
		} catch (InterruptedException e2) {
			e2.printStackTrace();
		} 
        finally{
        	//Close the socket
			try {
				this.socket.close();
			} catch (IOException e1) {
				e1.printStackTrace(); 
			}
			log.debug("ConnectionHandler finished");
        }
	}
}