/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.IOException;
import java.net.Socket;
import java.nio.channels.ServerSocketChannel;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;


/**
 * TODO: put some comment here!!
 *
 * @author Andrew C. Oliver <acoliver ot jboss dat org>
 */
public class Reaper implements Runnable {
    long pulse;
    private Map reapables;
    private boolean stop = false;
    public Reaper(long pulse) {
        this.pulse = pulse;
        this.reapables = new HashMap(50);
    }
    
    public void run() {
        try {
            while(stop == false) {
                    Thread.sleep(pulse);
                    long thetime = System.currentTimeMillis();
                    Iterator reapme = reapables.entrySet().iterator();
                    
                    while (reapme.hasNext()) {
                        TimeAndLife tal = (TimeAndLife)reapme.next();
                        tal.check(thetime);
                    }
            }
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * @param thread
     * @param timeout
     * @param life
     */
    public void register(Socket socket,Thread thread, long timeout, long life) {
        TimeAndLife tal = new TimeAndLife(socket, thread, timeout, life);
        tal.update(System.currentTimeMillis());
        reapables.put(thread,tal);
    }
    
    public void lock(Thread thread) {
        TimeAndLife tal = (TimeAndLife)reapables.get(thread);
        tal.lock();
    }
    
    public void unlock(Thread thread) {
        TimeAndLife tal = (TimeAndLife)reapables.get(thread);
        tal.unlock();
    }
    
    public void update(Thread thread) {
        TimeAndLife tal = (TimeAndLife)reapables.get(thread);
        tal.update(System.currentTimeMillis());
    }

	/**
	 * @param ssc
	 * @param thisthread
	 * @param timeout
	 * @param life
	 */
	public void register(ServerSocketChannel ssc, Thread thread, long timeout, long life) {
        TimeAndLife tal = new TimeAndLife(ssc, thread, timeout, life);
        tal.update(System.currentTimeMillis());
        reapables.put(thread,tal);
	}
    
   
}

class TimeAndLife {
    private long thistime;
    private long time;
    private long life;
    private Thread thread;
    private Socket socket;
    private boolean locked; //if we're processing a command we cannot timeout
	private ServerSocketChannel ssc;

    public TimeAndLife(Socket socket, Thread thread, long time, long life) {
        this.socket = socket;
        this.time = time;
        this.life = life;
        this.thread = thread;
    }
    
    /**
	 * @param ssc
	 * @param thread2
	 * @param timeout
	 * @param life2
	 */
	public TimeAndLife(ServerSocketChannel ssc, Thread thread, long time, long life) {
		this((Socket)null, thread, time, life);
		this.ssc = ssc;
	}

	public synchronized boolean check(long thetime) {
        boolean retval = false;
        //if we have timed out
        if(this.locked == false && ((thistime + time) > thetime)) {
            try {
                 if (socket == null) {
                 	ssc.close();
                 } else {
                 	socket.close();
                 }
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            thread.interrupt();
            retval = true;
        }
        
        //if our life is over
        if((life > thetime)) {
            try {
            		if (socket == null) {
            			ssc.close();
            		} else { 
            			socket.close();
            		}
            } catch (IOException e) {
                // TODO Auto-generated catch block
                e.printStackTrace();
            }
            thread.interrupt();
            retval = true;
        }
        return retval;
    }
    //we just executed a command so update
    public synchronized void update(long time) {
        thistime = time;
    }
    
    synchronized void lock() {
        this.locked = true;
    }
    
    synchronized void unlock() {
        this.locked = false;
    }
    
}