/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.OutputStream;
import java.io.IOException;
import java.io.PrintWriter;

/**
 * Response is a fragment of the command pattern.  Protocols have requests and responses.
 * For SMTP, the response is mostly just a transport between handlers.  For other protocols
 * this might have more meaning.
 * 
 * @author Andrew C. Oliver
 */
public interface Response  {

  /**
   * retrieve the request object associated with this response 
   * @return request object associated with the response
   */
  public Request getRequest();

  /**
   * retrieve the output stream for writing this response
   * @return output stream
   * @throws IOException in the event there is some problem in retreiving the outputstream
   */
  public OutputStream getOutputStream() throws IOException;

  /**
   * get a printwriter for the outputstream
   * @return printwriter
   * @throws IOException in the event a print writer could not be created
   */
  public PrintWriter getWriter() throws IOException;

  /**
   * Get the associated protocol instance
   * @return protocol instance associates with this response
   */
  public Protocol getProtocol();

  /**
   * Should this response terminate the conversation
   * @return boolean on whether to quit or not
   */
  public boolean isFinish();
}
