/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.IOException;
import java.net.Socket;
import java.nio.channels.ServerSocketChannel;

import EDU.oswego.cs.dl.util.concurrent.PooledExecutor;
import org.jboss.system.ServiceMBeanSupport;

/**
 * TODO: put some comment here!!
 *
 * @author Andrew C. Oliver <acoliver ot jboss dat org>
 */ 
public class ThreadPool 
             extends ServiceMBeanSupport
             implements ThreadPoolMBean {
    private int initial;
    private int min;
    private int max;
    private int idleKeepAlive;
    private PooledExecutor executor;
    private Thread reaperThread;
    private Reaper reaper;
    //todo:make configurable
    private long pulse = 1000;

    
    public ThreadPool() {
        executor = new PooledExecutor();
        reaper = new Reaper(pulse);        
    }
    
    public void startService() {
        updateParms();
        revise();
        reaperThread = new Thread(reaper);
        reaperThread.start();
    }
    
    public void stopService() {
        reaperThread.interrupt();
    }
    
    private void updateParms() {
        executor.setMaximumPoolSize(max);
        executor.setMinimumPoolSize(min);
        executor.setKeepAliveTime(idleKeepAlive);
    }
    
    private void revise() {
        if (initial > executor.getPoolSize()) {
            executor.createThreads(initial - executor.getPoolSize());
        }
    }
    
    public void handleConnection(Socket socket, String protocolName, long timeout, long life) throws InterruptedException {
        ConnectionHandler ch = new ConnectionHandler(socket, protocolName, timeout, life, reaper);
        executor.execute(ch);
    }
    
	/* (non-Javadoc)
	 * @see org.jboss.mail.ThreadPoolMBean#handleConnection(java.nio.channels.ServerSocketChannel, java.lang.String, long, long)
	 */
	public void handleConnection(ServerSocketChannel ssc, String protocolName, long timeout, long life) throws InterruptedException  {
		try {
			ConnectionHandler ch = new ConnectionHandler(ssc, protocolName, timeout, life, reaper);
			executor.execute(ch);
		} catch (IOException e) {
			log.warn("IOException in ThreadPool.handleConnection for NIO ", e);
		}
	}
    
    public void setInitial(int initial) {
        this.initial = initial;
    }
    
    public int getInitial() {
        return initial;
    }
    
    /**
     * @return Returns the max.
     */
    public int getMax() {
        return max;
    }
    /**
     * @param max The max to set.
     */
    public void setMax(int max) {
        this.max = max;
    }
    /**
     * @return Returns the min.
     */
    public int getMin() {
        return min;
    }
    /**
     * @param min The min to set.
     */
    public void setMin(int min) {
        this.min = min;
    }
    
    
    /**
     * @return Returns the idleKeepAlive.
     */
    public int getIdleKeepAlive() {
        return idleKeepAlive;
    }
    /**
     * @param idleKeepAlive The idleKeepAlive to set.
     */
    public void setIdleKeepAlive(int idleKeepAlive) {
        this.idleKeepAlive = idleKeepAlive;
    }



    
}
