/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;

/**
 * Client script runner works against a remote server/port running a script.
 */
public class ClientScriptRunner implements Runnable {
	private String host;
	private int port;
	private Socket sock;
	String start = null;
	Condition[] groups;
	public ClientScriptRunner(File script, String host, int port) throws Exception {
		FileInputStream stream = new FileInputStream(script);
		//BufferedInputStream bstream = new BufferedInputStream(stream);
		//BufferedReader rdr = new BufferedReader(new InputStreamReader(bstream));
		//this.start = rdr.readLine();
		this.sock = new Socket(host, port);
		this.groups = ScriptParser.parseScript(stream);
	}
	
	public void run() {
		try {
			InputStream is = sock.getInputStream();
			BufferedInputStream stream = new BufferedInputStream(is);
			BufferedReader rdr = new BufferedReader(new InputStreamReader(stream));
			OutputStream os = sock.getOutputStream();
			BufferedOutputStream bos = new BufferedOutputStream(os);
			PrintWriter writer = new PrintWriter(bos);
			
			for (int k = 0; k < this.groups.length; k++) {
				while (stream.available() > -1 && this.groups.length != k) {
					String line = rdr.readLine();
					System.out.println("line ="+line);
					String respond = groups[k].respond(line);
					System.out.println("response ="+respond);
					if (respond != null) {
						writer.println(respond);
						writer.flush();
					}
					k++;
				} 
			}
					
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	/**
	 * run in the same thread
	 * @param script to use
	 * @param host to connect to
	 * @param port to connect to
	 * @throws Exception if any problem happens
	 */
	public static void run(File script, String host, int port) throws Exception {
		ClientScriptRunner runner = new ClientScriptRunner(script, host, port);
		runner.run(); //single thread.
	}
	
	//pass in filename, host, port
	public static void main(String[] args) throws Exception { 
		String filename = args[0];
		String host = args[1];
		String port = args[2];
		run(new File(filename), host, Integer.parseInt(port));
	}
	
}
