/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Parses a test script into conditional response tree
 */
public class ScriptParser {
  
  public static Condition[] parseScript(InputStream stream) throws Exception {
  	List conditions = parseGroups(getGroups(getLines(stream)));
  	
	return (Condition[])conditions.toArray(new Condition[conditions.size()]);
  }
  
  private static List parseGroups(List groups) {
    List parsed = new ArrayList(groups.size());
    Iterator i = groups.iterator();
    while (i.hasNext()) {
    	List group = (List)i.next();
    	Condition c = new Condition(group);
    	parsed.add(c);
    }
  	return parsed;
  }
  
  private static List getGroups(List lines){
  	List groups = new ArrayList();
  	int groupStart = findNextGroup(lines,0);
  	int groupEnd = findEnd(lines,groupStart);
  	
  	while(groupStart < lines.size()) {
  		List group = new ArrayList(groupEnd - groupStart);
  		String lastLine = "";
	  	for (int k = groupStart; k < groupEnd; k++) {
	  		String line = (String)lines.get(k);
	  		if (line.trim().equals("")||!isPrefixed(line.trim())) {
	  			lastLine += line + "\r\n";
	  		} else {
	  			if (!lastLine.equals("")) {
	  				group.add(lastLine);
	  			}
	  			lastLine = line;
	  		}
	  	}
	  	group.add(lastLine);
	  	groups.add(group);
	  	groupStart = findNextGroup(lines,groupEnd+1);
	  	if (groupStart < lines.size()) {
	  		groupEnd = findEnd(lines,groupStart);
	  	}
  	}
    return groups;
  }
  
  /**
 * @param string
 * @return
 */
private static boolean isPrefixed(String line) {
	String[] subjects = line.split("\\:\\s");
	String subject = subjects[0];
	if (line.length() > 2 && subjects.length > 1 && line.charAt(subject.length()) == ':' &&
	    (subject.charAt(0) == 'S' || subject.charAt(0) == 'C' )
	   ) {
		return true;
	}
	return false;
}

private static int findEnd(List lines, int startAt) {
  	for (int k = startAt; k < lines.size(); k++) {
  		String line = (String) lines.get(k);
  		
  		if (line.trim().equals("END")) {
  			return k;
  		}
  	}
  	throw new RuntimeException("expected END before EOF");
  }
  
  private static int findNextGroup(List lines, int startAt) {
  	
  	for (int k = startAt; k < lines.size() -1; k++) {
  		String line = (String) lines.get(k);
  			
  			if (!line.trim().equals("")) {
  				return k;
  			}
  	}
  	return lines.size();
  }
  
  public static List getLines(InputStream stream) throws Exception {
  	List results = new ArrayList();
  	BufferedInputStream bstream = new BufferedInputStream(stream);

  	BufferedReader reader = new BufferedReader(new InputStreamReader(bstream));
  	while (reader.ready()) {
  		String line = reader.readLine();
  		results.add(line);
  	}
  	return results;
  }
}
