/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot;

import org.jboss.mail.dev.prot.inf.*;

import java.util.*;
import java.io.*;
import java.net.*;

import org.apache.log4j.*;

/**
 * abstract protocol testing manager
 *
 * @author mike andrews
 */
public abstract class AbstractManager implements MutableManager {

    private static final Logger logger = Logger.getLogger(AbstractManager.class);

    private String id;
    // map of id's to module factories
    private Map moduleFactories = new HashMap();
    private int port;
    private String host;
    private Socket socket;
    private BufferedReader reader;
    private Writer writer;
    private MyBuffer transcript = new MyBuffer();

    private static class MyBuffer {
	private StringBuffer buf = new StringBuffer();
	public MyBuffer append(String text) {
	    //logger.debug("APPEND: " + text);
	    buf.append(text);
	    return this;
	}
	public String toString() {
	    return buf.toString();
	}
    }
    
    public AbstractManager() {  }

    public void setId(String id) {
	this.id = id;
    }

    public void connect() throws IOException {
	socket = new Socket(host,port);
	reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	writer = new OutputStreamWriter(socket.getOutputStream());
    }

    public void disconnect() throws IOException {
	socket.close();
    }

    public boolean isConnected(){
	return !socket.isClosed();
    }
    
    public boolean didServerDisconnect(){
	throw new UnsupportedOperationException();
    }

    public String getFromServer() throws IOException {
	// this is not flexible but works for now with single-line responses
	String line = reader.readLine();
	transcript.append(line);
	return line;
    }
    
    public String getTranscript() {
	return transcript.toString();
    }

    public void sendToServer(String text) throws IOException {
	writer.write(text + "\r\n");
	writer.flush();
	transcript.append("\r\n" + text);
    }

    public String getId() {
	return id;
    }

    public void setPort(int port) {
	this.port = port;
    }

    public void setHost(String host) {
	this.host = host;
    }

    public String getHost() {
	return host;
    }

    public void addModuleFactory(ModuleFactory module) {
	moduleFactories.put(module.getId(),module);
	logger.debug("addModuleFactory("+module+")");
    }

    public int getPort() {
	return port;
    }

    public String[] getModuleFactoryIds() {
	String[] ids = new String[moduleFactories.size()];
	moduleFactories.values().toArray(ids);
	return ids;
    }

    public ModuleFactory getModuleFactory(String id) {
	return (ModuleFactory) moduleFactories.get(id);
    }

    public void close() throws IOException {
	if (!socket.isClosed()) {
	    socket.close();
	}
    }
}
