/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;
import org.jboss.mail.dev.prot.inf.Manager;
import org.jboss.mail.dev.prot.inf.Module;
import org.jboss.mail.dev.prot.inf.ModuleFactory;
import org.jboss.mail.dev.prot.inf.MutableModule;
import org.w3c.dom.Element;

/** 
 * @author <a href='mailto:mikea@xoba.com'>mike andrews</a>
 */	
public class ModuleFactoryBuilder {

    private static final Logger logger = Logger.getLogger(ModuleFactoryBuilder.class);

    public static ModuleFactory newModuleFactory(final Manager manager, final String id, final Class moduleClass) {
	return new ModuleFactory() {
		public String getId() {
		    return id;
		}
		public String toString() {
		    return getId();
		}

		public Module createModule(String patternText) 
		    throws InstantiationException {
			
		    try {
			Constructor cons = moduleClass.getConstructor(new Class[0]);
			MutableModule module = (MutableModule) cons.newInstance(new Object[0]);
			module.init(manager);
			module.setPatternText(patternText);
			module.setId(id);
			return module;
		    } catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		    } catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		    } catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		    }
		}
	    };	
    }

    public static ModuleFactory newModuleFactory(final Manager manager, final Element e) throws Exception {
	try {
	    String className = e.getAttribute("class");
	    ClassLoader cl = Thread.currentThread().getContextClassLoader();
	    Class clazz = cl.loadClass(className);
	    final Constructor cons = clazz.getConstructor(new Class[0]);

	    // check that module has all required fields
	    Element[] states = XmlUtilities.getElementsWithName(e,"state");
	    for (int i=0; i<states.length; i++) {
		String field =  states[i].getAttribute("field");
		try {
		    Method m = clazz.getMethod("get" + field,new Class[0]);
		} catch (NoSuchMethodException ex) {
		    logger.error("no field '"+field+"' in " + className);
		    throw ex;
		}
	    }

	    return new ModuleFactory() {
		    public String getId() {
			return e.getAttribute("id");
		    }

		    public String toString() {
			return getId();
		    }

		    public Module createModule(String patternText) throws InstantiationException {
			try {
			    MutableModule module = (MutableModule) cons.newInstance(new Object[0]);
			    module.init(manager);
			    module.setPatternText(patternText);
			    module.setId(e.getAttribute("id"));
			    return module;
			} catch (InvocationTargetException e) {
			    throw new RuntimeException(e);
			} catch (IllegalAccessException e) {
			    throw new RuntimeException(e);
			}
		    }
		};
	} catch (ClassNotFoundException ex) {
	    logger.error(ex);
	    throw ex;
	} catch (NoSuchMethodException ex) {
	    logger.error(ex);
	    throw ex;
	}
    }

}
