/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot;

import org.apache.log4j.Logger;
import org.jboss.mail.dev.prot.inf.Conditional;
import org.jboss.mail.dev.prot.inf.ConditionalFactory;
import org.jboss.mail.dev.prot.inf.Manager;
import org.jboss.mail.dev.prot.inf.ModuleFactory;
import org.jboss.mail.dev.prot.inf.Script;
import org.w3c.dom.Element;

/**
 * represents a script to be run
 *
 * @author mike andrews
 */
public class ScriptImpl implements Script {
    
    private static final Logger logger = Logger.getLogger(ScriptImpl.class);

    private Element e;
    private Manager manager;
    private ConditionalFactory[] conditionalFactories = new ConditionalFactory[0];
    private Conditional[] conditionals;
    private String status;

    public ScriptImpl(Element e, Manager manager) {
	this.e = e;
	this.manager = manager;
    }

    public String getDescription() {
	Element[] elements = XmlUtilities.getElementsWithName(e,"description");
	return XmlUtilities.getElementText(elements[0]);
    }
    
    public ScriptImpl(Element e, Manager manager, ConditionalFactory[] conditionalFactories) {
	this.e = e;
	this.manager = manager;
	this.conditionalFactories = conditionalFactories;
    }

    public void setConditionalFactories(ConditionalFactory[] conditionalFactories) {
	this.conditionalFactories = conditionalFactories;	
    }

    public ModuleFactory getDefaultClientModule() {
	return manager.getModuleFactory(e.getAttribute("client-module"));
    }

    public ModuleFactory getDefaultServerModule() {
	return manager.getModuleFactory(e.getAttribute("server-module"));
    }

    private Conditional[] getConditionals() {
	if (conditionals == null) {
	    try {
		conditionals = new Conditional[conditionalFactories.length];
		for (int i=0; i<conditionals.length; i++) {
		    //logger.debug("going to create conditional... " + conditionalFactories[i]);
		    conditionals[i] = conditionalFactories[i].createConditional();
		}
	    } catch (InstantiationException e) {
		throw new RuntimeException(e);
	    }
	}
	return conditionals;
    }

    public String getId() {
	return e.getAttribute("id");
    }

    public Role getRole() {
	return Role.getRoleWithName(e.getAttribute("role"));
    }

    public Manager getManager() {
	return manager;
    }

    /**
     * here is the meat of the scrip, kind-of like a 'run' method. all conditionals must
     * evaluate to true in sequence. once the script is run, the value is set.
     */
    public boolean getValue() {
	conditionals = getConditionals();
	for (int i=0; i<conditionals.length; i++) {
	    if (!conditionals[i].getValue()) {
		status = conditionals[i].getStatus();
		return false;
	    }
	}
	return true;
    }

    public String getStatus() {
	if (getValue()) {
	    return "OK";
	} else {
	    return (status == null ? "" : "STATUS: " + status + "\n") + manager.getTranscript();
	}
    }
}
