/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.jboss.mail.dev.prot.inf.ConditionalFactory;
import org.jboss.mail.dev.prot.inf.ManagerFactory;
import org.jboss.mail.dev.prot.inf.Script;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/** 
 * @author <a href='mailto:mikea@xoba.com'>mike andrews</a>
 */	
public class ScriptParser {

    private static final Logger logger = Logger.getLogger(ScriptParser.class);

    public static Script[] parseXml(InputSource source) throws IOException, SAXException {

	List scriptList = new Vector();

	Document doc = XmlUtilities.parse(source);

	Element e = doc.getDocumentElement();

	String defaultHost = e.getAttribute("default-host");
	
	Map managerFactoryMap = new HashMap();

	{
	    Element[] managers = XmlUtilities.getElementsWithName(e,"manager");
	    
	    for (int i=0; i<managers.length; i++) {
		String id = managers[i].getAttribute("id");
		try {
		    ManagerFactory fact = new ManagerFactoryImpl(managers[i],defaultHost);
		    managerFactoryMap.put(id,fact);
		} catch (Exception ex) {
		    logger.error("can't create manager with id: '"+id+"': " + ex);
		}
	    }
	}
	
	{
	    Element[] scripts = XmlUtilities.getElementsWithName(e,"script");
	    for (int i=0; i<scripts.length; i++) {

		// a script is basically a list of conditionals that need to be 
		// executed in order. conditionals, in turn, are composite
		List conditionalFactories = new Vector();
			
		ManagerFactory manager = (ManagerFactory) managerFactoryMap.get(scripts[i].getAttribute("manager"));
		String id = scripts[i].getAttribute("id");
		String serverModule = scripts[i].getAttribute("server-module");
		logger.debug("script: " + id);
		ScriptImpl script = new ScriptImpl(scripts[i],manager.createManager());
		ConditionalFactory[] factories = ConditionalFactoryBuilder.parseConditionalElementChildren(scripts[i],script);
		script.setConditionalFactories(factories);
		scriptList.add(script);
	    }
	}
	
	Script[] array = new Script[scriptList.size()];
	scriptList.toArray(array);
	return array;
    }

}
