/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot.inf;

import java.io.*;

/**
 * protocol testing manager. this is actually more like a 'protocol' than 'manager'
 *
 * @author mike andrews
 */
public interface Manager {
    
    public String getId();

    public int getPort();

    public String getHost();

    public String[] getModuleFactoryIds();

    /**
     * gets the module factory for the module of given id
     */
    public ModuleFactory getModuleFactory(String id);

    /**
     * connects to the server, throws IOException if problems connecting
     */
    public void connect() throws IOException;

    public void disconnect() throws IOException;

    public boolean isConnected();
    
    public boolean didServerDisconnect();

    /**
     * gets next text from server
     */
    public String getFromServer() throws IOException;
    
    /**
     * puts next text command or response to server
     */
    public void sendToServer(String text) throws IOException;

    /**
     * to close all resources associated with manager
     */
    public void close() throws IOException;

    /**
     * gets verbatim transcript of conversation
     */
    public String getTranscript();
}
