/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev.prot.inf;

import java.util.*;

/**
 * represents a script to be run
 *
 * @author <a href='mailto:mikea@xoba.com>mike andrews</a>
 */
public interface Script extends Conditional {

    public String getId();

    public Role getRole();

    public Manager getManager();

    public ModuleFactory getDefaultClientModule();
    public ModuleFactory getDefaultServerModule();

    public String getDescription();

    /**
     * type-safe enumeration of script roles
     */
    public static class Role {
	
	private static Map roles = new HashMap();

	public static final Role CLIENT = new Role("client");
	public static final Role SERVER = new Role("server");
	public static final Role NONE = new Role("none");

	private String name;

	private Role(String name) { this.name = name; roles.put(name,this); }

	public static Role getRoleWithName(String name) {
	    if (roles.containsKey(name)) {
		return (Role) roles.get(name);
	    } else {
		return NONE;
	    }
	}

	public String toString() { return name; }
    }

}
