/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.domaingroup;


import org.jboss.mail.ServiceProxyFactory;
import org.jboss.system.ServiceMBean;
import org.w3c.dom.Element;

/**
 * represents a list of domains
 * @author Andrew C. Oliver
 */
public interface DomainGroupMBean extends ServiceMBean, ServiceProxyFactory {

    /**
     * adds a domain to the group
     *
     * @param string the domain to add to domain group
     */
    public void add(String string);

    /**
     * lists the domains explicitly added to the group; 
     * this method should not be used for testing membership since other domains
     * not in the array may test positive for being in group too.
     *
     * @return array of all domains (character-case irrelevant)
     */
    public String[] listDomains();

    /**
     * tests whether or not a domain is part of the group, including all local
     * internet address domains too as required by rfc 1123-5.2.17. 
     * (example: the domain string "[127.0.0.1]" will always be in group, etc...)
     *
     * @param the domain name
     * @return whether or not the domain is in the group. domains are case insensitive
     */
    public boolean isInGroup(String string);

    /**
     * @return whether or not the group implicitly includes local network interface literals; i.e.,
     * domains like '[127.0.0.1]' or '[64.32.179.43]' etc
     */
    public boolean getIncludesLocalInterfaces();

    /**
     * clears and initializes the group to have the domains in the xml element. top-level element
     * should be 'domains' with an optional attribute 'includes-local-interfaces' whose value
     * is either 'true' or 'false'. if true, then 'getIncludesLocalInterfaces()' method will
     * return true. child elements of 'domains' elements are 'domain' elements, each of which
     * whose text content is the name of a domain part of the group.
     *
     * @param element the xml element containing domain configuration information
     */
    void setDomains(Element element);
}
