/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.mailbox;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.jboss.system.ServiceMBeanSupport;
import org.jboss.system.ServiceMBean;
import org.jboss.mail.util.JBMailJMXUtil;

/**
 * Provides simplified access to the MailboxMBean instance
 * in the current environment.
 *
 * @author Eric Daugherty
 */
public class Mailbox
    extends ServiceMBeanSupport
    implements ServiceMBean, MailboxMBean {

    private MailboxManager mailboxManager;

    /**
     * Called on startup by the container.  The implementationClassName should
     * be configured in the jboss-service.xml.
     *
     * @param implementationClassName the full name of the MailboxManager impelementation.
     * @throws Exception thrown if the class could not be found or created.
     */
    public void setImplementationClass( String implementationClassName ) throws Exception
    {
        Class mailboxManagerClass = Class.forName( implementationClassName );
        mailboxManager = (MailboxManager) mailboxManagerClass.newInstance();
    }

    /**
     * Provides access to the MailboxManager implementation.
     *
     * @return MailboxManager implementation
     */
    public MailboxManager getMailboxManagerInstance()
    {
        return mailboxManager;
    }


    /**
     * Static helper method to provide easy access to the MailboxManager implementation.
     *
     * @return MailboxManager implementation
     * @throws Exception thrown if there is an error looking up the MailboxMBean
     */
    public static MailboxManager getMailboxManager() throws Exception
    {
        MBeanServer server = JBMailJMXUtil.locateJBoss();
        ObjectName mailboxMBeanName = new ObjectName( "jboss.mail:type=MailServices,name=Mailbox" );

        return (MailboxManager)
            server.getAttribute( mailboxMBeanName, "MailboxManagerInstance" );
    }
}
