/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.mailbox.entity.message;

import java.rmi.RemoteException;
import javax.ejb.EntityBean;
import javax.ejb.EJBException;
import javax.ejb.RemoveException;
import javax.ejb.EntityContext;
import javax.ejb.CreateException;

import org.apache.log4j.Logger;

import org.jboss.mail.mailbox.entity.EntityMessage;

/**
 * Implements an Entity bean store for the MailboxMessage data.
 *
 * @author Eric Daugherty
 * @author Andrew C. Oliver (performance optimzations only)
 */
public abstract class MessageBean implements EntityBean {

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( MessageBean.class );
 
    private EntityContext ctx;

    private EntityMessage msg;

    public MessageBean() {
        log.debug( "MessageBean Entity EJB Created.");
    }


    public String ejbCreate( String id ) throws CreateException {
        this.setId( id );
        return null;
    }

    public void ejbPostCreate( String id ) throws CreateException {
    }

    public void ejbActivate() throws EJBException, RemoteException {
    }

    public void ejbLoad() throws EJBException, RemoteException {
           log.warn("ejbLoad"+getId());
        try {
           this.msg = new EntityMessage(getId(),getHeaders(),getBody());
        } catch (Exception e) {
           log.error("couldn't create EntityMessage instance"+e,e);
        }
    }

    public void ejbPassivate() throws EJBException, RemoteException {
    }

    public void ejbRemove() throws RemoveException, EJBException, RemoteException {
    }

    public void ejbStore() throws EJBException, RemoteException {
        this.msg = new EntityMessage(getId(),getHeaders(),getBody());
    }


    public void setEntityContext(EntityContext ctx)
                      throws EJBException,
                             RemoteException {
      this.ctx=ctx;
    }

    public void unsetEntityContext() throws EJBException, RemoteException {
      this.ctx = null;
    }

    public EntityMessage getMessage() {
      if (msg == null) {
        this.msg = new EntityMessage(getId(),getHeaders(),getBody());
      } 
      return this.msg;
    }

    public abstract String getId();

    public abstract void setId( String id );

    public abstract String getFolderName();

    public abstract void setFolderName( String folderName );

    public abstract String getHeaders();

    public abstract void setHeaders( String headers );

    public abstract String getBody();

    public abstract void setBody( String body );
}
