package org.jboss.mail.mailbox.entity.message;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.Context;
import javax.naming.NamingException;

import java.util.*;

import org.jboss.mail.message.Mail;
import org.jboss.mail.mailbox.entity.EntityMessage;
import org.jboss.logging.Logger;

/**
   Session bean implementation for the Catalogue.
   It stores a list of Item objects, information on
   which can be retrieved by a client using the business methods.
 */
public class MessageSB implements SessionBean {

   public void setSessionContext(SessionContext context) {}
   public void ejbRemove() {}
   public void ejbCreate() {}
   public void ejbActivate() {}
   public void ejbPassivate() {}

   private static final Logger log = Logger.getLogger( MessageSB.class );
   private static Random keyGen = new Random( System.currentTimeMillis()  );

   public EntityMessage[] getMessagesForFolder(String folderName)  {
     EntityMessage[] mailboxMessages = null;
     try {
            InitialContext context = new InitialContext();
            LocalMessageHome messageHome = (LocalMessageHome) context.lookup( "jbossmail.EntityMessage" );

            Collection messages = messageHome.findByFolderName( folderName );
            int index = 0;
            mailboxMessages = new EntityMessage[messages.size()];
            Iterator messageIterator = messages.iterator();
            while( messageIterator.hasNext() )
            {
                LocalMessage message = (LocalMessage) messageIterator.next();
                log.debug( "Message ID="+message.getId() );
                if (!message.getFolderName().trim().equals(folderName.trim())) {
                    log.warn("WEIRD requested folder "+folderName+" but got "+message.getFolderName() + 
			     " ("+"Message ID="+message.getId()+")");
                }
                EntityMessage msg = message.getMessage();
                msg.setLocal(message);
                mailboxMessages[index++] = msg;
            }
        } catch( NamingException namingException )
        {
            log.error( "Error loading messages for mailbox: " + folderName + ".  Unable to lookup Entity Bean Home: " + namingException, namingException );
            mailboxMessages = new EntityMessage[0];
        }
        catch( FinderException e ) {
            log.error( "Error loading messages for mailbox: " + folderName + " " + e, e );
            mailboxMessages = new EntityMessage[0];
        }
         return mailboxMessages;
   }

  public void deleteMessages(EntityMessage[] messages) {
        EntityMessage message;
        for( int index = 0; index < messages.length; index++ )
        {
            message = messages[index];
            if( message.isDeleted() )
            {
                message.delete();
            }
        }
  }

 public void createMessage (String folderName, Mail message) {
    try {
            InitialContext context = new InitialContext();
            LocalMessageHome messageHome = (LocalMessageHome) context.lookup( "jbossmail.EntityMessage" );
            // MIKEA: keep trying to create bean until unique key is found, in
            // unlikely case there is a random primary key collision (probability
            // of collision will increase linearly with the number of stored messages)
            // ACO: TODO make this use sequences ala JBossCMP. MIKEA: good idea!
            //
            LocalMessage messageBean = null;
            while (messageBean == null) {
                try {
                    messageBean = messageHome.create( String.valueOf( keyGen.nextLong() ) );
                } catch (DuplicateKeyException e) {
                    // keep trying until we generate a unique key
                    log.debug("encountered duplicate key! retrying");
                }
            }

            messageBean.setFolderName( folderName );

            // Convert headers to a string.
            Iterator headerEnumeration = message.getMailHeaders().getAllHeaderLines();
            StringBuffer headers = new StringBuffer();
            boolean first = true;
            while( headerEnumeration.hasNext() )
            {
                if( !first ) headers.append( "\r\n" );
                first = false;
                headers.append( (String) headerEnumeration.next() );
            }

            messageBean.setHeaders( headers.toString() );

            // Convert Body to a String.
            messageBean.setBody( new String( message.getBody() ) );
        }
        catch( NamingException namingException )
        {
            log.error( "Error storing messages for mailbox: " + folderName + ".  Unable to lookup Entity Bean Home: " + namingException, namingException );
        }
        catch (CreateException createException) {
            log.error( "Error storing new message for mailbox: " + folderName + ".  Create Exception: " + createException, createException );
        }


 }

}

