/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.mailbox.memory;

import org.jboss.mail.mailbox.MailboxManager;
import org.jboss.mail.mailbox.Folder;

import java.util.HashMap;

/**
 * Simple MailboxManger implementation that stores all
 * information in memory only.  If an unknown mailbox is requested,
 * a new (empty) mailbox will be created and returned.
 *
 * @author Eric Daugherty
 */
public class MemoryMailbox implements MailboxManager {

    // Stores the mailboxes.
    private HashMap mailboxes = new HashMap();

    /**
     * Returns a MemoryFolder implemenation of the Folder interface.
     * If the username does not exist, a new Folder is created.
     *
     * @param username the username to load the mailbox for.
     * @return a MemoryFolder instance.
     */
    public synchronized Folder getMailbox(String username) {
        Folder folder = (Folder) mailboxes.get( username );
        if( folder == null )
        {
            folder = new MemoryFolder();
            mailboxes.put( username, folder );
        }

        return folder;
    }
}
