/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.mailhandler.nukesposter;

import java.util.ArrayList;

import javax.ejb.MessageDrivenBean;
import javax.jms.MessageListener;
import javax.jms.ObjectMessage;
import javax.mail.internet.MimeUtility;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.jboss.mail.MailListener;
import org.jboss.mail.message.Mail;
import org.jboss.mail.message.MailAddress;
import org.jboss.mail.message.StandardMailHeaders;
import org.jboss.mail.util.JBMailJMXUtil;
import org.jboss.mail.util.MimeUtil;

/**
 * Processes messages from the local delivery queue 
 * and attempts posting them to the relvant Nukes forum. 
 * If the forum/topic cannot be found a relevant mail is generated 
 * for the sender.
 *
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
 */
public class NukesPosterMessageDrivenBean implements MessageDrivenBean, MessageListener {

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( NukesPosterMessageDrivenBean.class );
    private Context ctx;
    /** Name of the MailListener MBean */
    private MailListener listener;
    ObjectName nukesPosterName;
    
    /**
     * Initialize the MessageDrivenBean
     */
    public NukesPosterMessageDrivenBean () {

        log.debug( "NukesPosterMessageDrivenBean initialized." );
    }

    /**
     * Handles incoming email messages from the queue and delivers them
     * to the appropriate nukes forum
     *
     * @param jmsMessage the incoming JMS Message
     */
    public void onMessage( javax.jms.Message jmsMessage ) {
        
        ArrayList failed = new ArrayList();
        try {
            log.debug("Received message");
            ObjectMessage jmsObjectMessage = (ObjectMessage) jmsMessage;
            Object messageObject = jmsObjectMessage.getObject();
            if( !(messageObject instanceof Mail) ){
                log.error( "Invalid Object placed in Local Delivery queue.  Expected type is: " + Mail.class + " but was: " + messageObject.getClass() + "; Message will be ignored.");
                return;
            }

            Mail mail = (Mail)messageObject;
            MBeanServer server = JBMailJMXUtil.locateJBoss();
            MailAddress[] toAddrs = mail.getTo(); 
            
            for (int i = 0 ; i < toAddrs.length ; i++){
            	
                String to = toAddrs[0].getUser();
                String subject = MimeUtility.decodeText(mail.getMailHeaders().getHeader(StandardMailHeaders.SUBJECT)[0]);
                String body = MimeUtil.getPlainTextBody(mail);
                boolean sent = ((Boolean)server.invoke(nukesPosterName, "postMessage", new Object[]{to, subject, body}, new String[]{"java.lang.String", "java.lang.String", "java.lang.String"})).booleanValue();
               
                if (!sent){
                    //TODO: The user is either blocked, or they are flooding
                    //Flooding can be triggered for legitimate reasons, so 
                    //we should probably keep track of who/when and for the legitimate 
                    //ones generate a bounce mail.
                    log.debug("Message not posted due to flooding or user blocked.");
                }
            }
                   
        }
        catch ( Exception e ) {
            log.error( "Error handling message, message lost! " + e, e );
            //TODO: Generate notification email
        }
    }

    public void setMessageDrivenContext(javax.ejb.MessageDrivenContext context ) {
    }

    public void ejbCreate() {
        try {
            ctx = new InitialContext();
            ctx = (Context) ctx.lookup("java:comp/env");
        } catch (NamingException e) {
            log.error("Could not get initial context", e);
            throw new RuntimeException("Could not get initial context!");
        }
        try {
            listener = (MailListener)JBMailJMXUtil.getService(new ObjectName((String)ctx.lookup("MailListenerService")));
            
        } catch (Exception e) {
            log.error("Could not get the JMX name of the MailListener service (*:MailListener:* usually) "+ e, e);
            throw new RuntimeException("Could not get the JMX name of the delivery service");
        }
    
        try {
            nukesPosterName = new ObjectName((String)ctx.lookup("NukesForumMailPosterService"));
        } catch (Exception e) {
            log.error("Could not get the JMX name of the NukesForumMailPoster service (*:NukesForumMailPosterService:* usually) "+ e, e);
            throw new RuntimeException("Could not get the JMX name of the nukes forum mail poster service");
        }
    }

    public void ejbRemove() {
    }


}

