/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist;

import org.jboss.mail.message.MailAddress;

/** Defines the available operations that can be made to an individual mailing list
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public interface MailList {
	
	/** Gets the address of the list 
	 * @return The address if the list
	 */
	MailAddress getListAddress();
	
	/** Gets the members of the list 
	 * @return a list of the members of the list
	 */
	MailAddress[] getMembers();
	
	/** Adds an address to the list
	 * @param address Address to be added 
	 */
	void addMember(MailAddress address);
	
	/** Checks if an address exists in the list
	 * @param address Address to be added
	 * @return true if is in list
	 */
	boolean isMember(MailAddress address);
	
	/** Removes an address from the list 
	 * @param address Address to be removed
	 */
	void removeMember(MailAddress address);
	
	/** Gets the properties of the list - a clone should be returned
	 * @return a clone of the MailListProperties for the list
	 */
	MailListProperties getProperties();
	
	/** Sets the properties of the list
	 * @param properties properties to be set
	 */
	void setProperties(MailListProperties properties);
}
