/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist;

import org.jboss.mail.message.MailAddress;

/** Defines available operations for managing MailingLists
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public interface MailListManager {
	
	/** Creates a MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to create
	 * @param properties of list we are creating
	 * @return false if a list already exists with that name
	 */
	boolean createList(MailAddress listAddress, MailListProperties properties);

	/** Deletes the MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to delete
	 */
	void deleteList(MailAddress listAddress);

	/** Finds the MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to find
	 * @return The created list. If the list already existed the existing list is returned
	 */
	MailList findList(MailAddress listAddress);
}
