package org.jboss.mail.maillist;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.jboss.mail.ServiceProxy;
import org.jboss.mail.message.MailAddress;
import org.jboss.mail.util.JBMailJMXUtil;

/** ServiceProxy used to call methods on the MailListManager MBean
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public class MailListManagerProxy implements ServiceProxy, MailListManager {
	ObjectName on;
	MBeanServer srv;

	/**
	 * constructor used by the AbstractListServ subclass mbeans to pass in their ObjectName instance 
	 * @param objectname of the calling ListServ MBeans (for a callback)
	 */    
	public MailListManagerProxy(ObjectName objectname) {
		   srv = JBMailJMXUtil.locateJBoss();
		   on = objectname;                    
	}

	
	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.MailListManager#createList(org.jboss.mail.message.MailAddress, org.jboss.mail.maillist.MailListProperties)
	 */
	public boolean createList(
		MailAddress listAddress,
		MailListProperties properties) {
			
		try{
			Object obj =  srv.invoke(on, 
							"createList", 
							new Object[]{listAddress, properties},
							new String[]{"org.jboss.mail.message.MailAddress", 
									"org.jboss.mail.maillist.MailListProperties"});
									
			return ((Boolean)obj).booleanValue();
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.MailListManager#deleteList(org.jboss.mail.message.MailAddress)
	 */
	public void deleteList(MailAddress listAddress) {
			
		try{
			Object obj =  srv.invoke(on, 
							"deleteList", 
							new Object[]{listAddress},
							new String[]{"org.jboss.mail.message.MailAddress"});									
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.MailListManager#findList(org.jboss.mail.message.MailAddress)
	 */
	public MailList findList(MailAddress listAddress) {
			
		try{
			Object obj =  srv.invoke(on, 
							"findList", 
							new Object[]{listAddress},
							new String[]{"org.jboss.mail.message.MailAddress"});
									
			return (MailList)obj;
		} catch (Exception e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
	}

}
