/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
 package org.jboss.mail.maillist;

import java.util.HashMap;

/** Used to get/set properties from a mailing list. It is initialised with 
 * default values that hopefully make sense!  
 *
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
 */
public class MailListProperties implements MailListPropertyConstants, Cloneable {
	
	static HashMap defaultProperties = new HashMap();
	static{
		defaultProperties.put(REPLY_TO_LIST, new Boolean(true));
		defaultProperties.put(PREFIX_AUTO_BRACKETED, new Boolean(true));
		defaultProperties.put(ATTACHMENT_ALLOWED, new Boolean(false));
		defaultProperties.put(MEMBERS_ONLY, new Boolean(true));
	}
	
	private HashMap properties;
	
	public MailListProperties(){
		properties = (HashMap)defaultProperties.clone();
	}
	
	public void setProperty(String name, String value){
		properties.put(name, value);
	}
	
	public void setProperty(String name, Long value){
		properties.put(name, value);		
	}
	
	public void setProperty(String name, Boolean value){
		properties.put(name, value);
		
	}
	
	public String getProperty(String name){
		String val = (String)properties.get(name);
		
		if (val == null){
			val = (String)defaultProperties.get(name);
		}
		 
		return val;
	}

	public Boolean getPropertyBool(String name){
		Boolean val = (Boolean)properties.get(name);
		
		if (val == null){
			val = (Boolean)defaultProperties.get(name);
		}
		 
		return val;
	}

	public Long getPropertyLong(String name){
		Long val = (Long)properties.get(name);
		
		if (val == null){
			val = (Long)defaultProperties.get(name);
		}
		 
		return val;
	}
	
	public Object clone(){
		MailListProperties mlprops = new MailListProperties();
		mlprops.properties = (HashMap)properties.clone();
		return mlprops;
	}
}
