/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist.entity;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.mail.maillist.MailList;
import org.jboss.mail.maillist.MailListProperties;
import org.jboss.mail.maillist.entity.bean.MailListSBLocal;
import org.jboss.mail.maillist.entity.bean.MailListSBLocalHome;
import org.jboss.mail.message.MailAddress;

/** Entity implementation of a MailList
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public class EntityMailList implements MailList {
	
	MailAddress listAddress;
	MailListProperties properties;
	
	public EntityMailList(MailAddress listAddress, MailListProperties properties){
		this.listAddress = listAddress;
		this.properties = properties;
	}

	/** Adds an address to the list
	 * @param address Address to be added 
	 */
	public void addMember(MailAddress address) {
		
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			mailListSBLocal.addMember(listAddress, address);
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (CreateException e) {
			throw new RuntimeException(e);
		}		
	}

	/** Checks if an address exists in the list
	 * @param address Address to be added
	 * @return true if is in list
	 */
	public boolean isMember(MailAddress address) {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			return mailListSBLocal.isMember(listAddress, address);
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (CreateException e) {
			throw new RuntimeException(e);
		}		
	}

	/** Gets the address of the list 
	 * @return The address if the list
	 */
	public MailAddress getListAddress() {
		return listAddress;
	}

	/** Gets the members of the list 
	 * @return a list of the members of the list
	 */
	public MailAddress[] getMembers() {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			return mailListSBLocal.getMembers(listAddress);
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (CreateException e) {
			throw new RuntimeException(e);
		}		
	}

	/** Removes an address from the list 
	 * @param address Address to be removed
	 */
	public void removeMember(MailAddress address) {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			mailListSBLocal.removeMember(listAddress, address);
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (CreateException e) {
			throw new RuntimeException(e);
		}		
	}

	/** Gets the properties of the list - a clone should be returned
	 * @return a clone of the MailListProperties for the list
	 */
	public MailListProperties getProperties() {
		return properties;
	}

	/** Sets the properties of the list
	 * @param properties properties to be set
	 */
	public void setProperties(MailListProperties properties) {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			mailListSBLocal.setProperties(listAddress, properties);
		} catch (NamingException e) {
			throw new RuntimeException(e);
		} catch (CreateException e) {
			throw new RuntimeException(e);
		}		
	}

}
