/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist.entity;

import javax.ejb.CreateException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.mail.ServiceProxy;
import org.jboss.mail.ServiceProxyFactory;
import org.jboss.mail.maillist.MailList;
import org.jboss.mail.maillist.MailListManagerProxy;
import org.jboss.mail.maillist.MailListProperties;
import org.jboss.mail.maillist.MailListPropertyConstants;
import org.jboss.mail.maillist.entity.bean.MailListSBLocal;
import org.jboss.mail.maillist.entity.bean.MailListSBLocalHome;
import org.jboss.mail.message.MailAddress;
import org.jboss.system.ServiceMBeanSupport;

/** Entity implementation of MailListManager MBean
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public class EntityMailListManager extends ServiceMBeanSupport 
									implements EntityMailListManagerMBean, 
												ServiceProxyFactory,
												MailListPropertyConstants {
													
	
	public EntityMailListManager(){
												
	}
	/** Creates a MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to create
	 * @param properties of list we are creating
	 * @return false if a list already exists with that name
	 */
	public boolean createList(MailAddress listAddress, MailListProperties properties) {
		
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			return mailListSBLocal.createList(listAddress,properties);
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return false;
	}

	/** Deletes the MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to delete
	 */
	public void deleteList(MailAddress listAddress) {
		
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			mailListSBLocal.deleteList(listAddress);
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/** Finds the MailList with the passed in address as its name
	 * 
	 * @param listAddress Address of list to find
	 * @return The created list. If the list already existed the existing list is returned
	 */
	public MailList findList(MailAddress listAddress) {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			return mailListSBLocal.findList(listAddress);
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	/* (non-Javadoc)
	 * @see org.jboss.mail.ServiceProxyFactory#produceServiceProxy()
	 */
	public ServiceProxy produceServiceProxy() {
		// TODO Auto-generated method stub
		return new MailListManagerProxy(this.getServiceName());
	}

	/** Just here to test in-container via jmx-console.
	 * To be removed once we are able to test using junit
	 */
	public void test(){
		log.debug("Getting service proxy");
		MailListManagerProxy proxy = (MailListManagerProxy)produceServiceProxy();
		MailAddress listAddr1 = MailAddress.parseSMTPStyle("list1@nono.com");
		
		log.debug("finding list1");
		MailList list1 = proxy.findList(listAddr1);
		log.debug("Found list1: " + list1);
		
		MailListProperties props = new MailListProperties();
		
		log.debug("creating list1");
		proxy.createList(listAddr1, props);		
		log.debug("finding list1");
		list1 = proxy.findList(listAddr1);
		log.debug("Found list1: " + list1);

		MailAddress listAddr2 = MailAddress.parseSMTPStyle("list2@nono.com");
		log.debug("creating list2");
		proxy.createList(listAddr2, props);

		log.debug("finding list1");
		list1 = proxy.findList(listAddr1);
		log.debug("Found list1: " + list1);

		log.debug("finding list2");
		MailList list2 = proxy.findList(listAddr2);
		log.debug("Found list2: " + list1);


		
		log.debug("Getting members...");
		MailAddress[] members = list1.getMembers();		
		log.debug("finding list1:" + members.length);
		
		log.debug("isMember");
		boolean rslt = list1.isMember(MailAddress.parseSMTPStyle("no@body.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("remove non-exist member");
		list1.removeMember(MailAddress.parseSMTPStyle("no@body.com"));
		
		log.debug("set props");
		props.setProperty(MailListPropertyConstants.MEMBERS_ONLY, new Boolean(false));
		list1.setProperties(props);
		log.debug("set props - done");
		
		log.debug("adding members");
		list1.addMember(MailAddress.parseSMTPStyle("one@somewhere.com"));
		list1.addMember(MailAddress.parseSMTPStyle("two@somewhere.com"));
		list1.addMember(MailAddress.parseSMTPStyle("three@somewhere.com"));

		list2.addMember(MailAddress.parseSMTPStyle("nothing@special.com"));
		list2.addMember(MailAddress.parseSMTPStyle("three@somewhere.com"));

		
		log.debug("adding existing member");
		list1.addMember(MailAddress.parseSMTPStyle("one@somewhere.com"));
		
		log.debug("remove non-exist member");
		list1.removeMember(MailAddress.parseSMTPStyle("no@body.com"));

		log.debug("Getting members...");
		members = list1.getMembers();
		for (int i = 0 ; i < members.length ; i++){		
			log.debug("member:" + members[i]);
		}

		log.debug("remove member");
		list1.removeMember(MailAddress.parseSMTPStyle("three@somewhere.com"));

		members = list1.getMembers();
		for (int i = 0 ; i < members.length ; i++){		
			log.debug("member:" + members[i]);
		}
		
		log.debug("isMember (yes)");
		rslt = list1.isMember(MailAddress.parseSMTPStyle("two@somewhere.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("isMember (no)");
		rslt = list1.isMember(MailAddress.parseSMTPStyle("three@somewhere.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("isMember (yes)");
		rslt = list2.isMember(MailAddress.parseSMTPStyle("nothing@special.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("isMember (yes)");
		rslt = list2.isMember(MailAddress.parseSMTPStyle("three@somewhere.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("isMember (no)");
		rslt = list2.isMember(MailAddress.parseSMTPStyle("two@somewhere.com"));
		log.debug("isMember: " + rslt);
		
		log.debug("deleting lists");
		proxy.deleteList(listAddr1);
		proxy.deleteList(listAddr2);
		log.debug("deleted lists");
	}
	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.entity.EntityMailListManagerMBean#createList(org.jboss.mail.message.MailAddress, boolean, boolean, boolean, boolean)
	 */
	public boolean createList(String listAddress, boolean reply_to_list, 
			                                           String subject_prefix, 
			                                           boolean prefix_auto_bracketed, 
													   boolean attachment_allowed, 
													   boolean members_only) {
		MailListProperties props = new MailListProperties();
		props.setProperty(MailListPropertyConstants.REPLY_TO_LIST, new Boolean(reply_to_list));
		props.setProperty(MailListPropertyConstants.PREFIX_AUTO_BRACKETED,new Boolean(prefix_auto_bracketed));
		props.setProperty(MailListPropertyConstants.ATTACHMENT_ALLOWED, new Boolean(attachment_allowed));
		props.setProperty(MailListPropertyConstants.MEMBERS_ONLY, new Boolean(members_only));
		props.setProperty(MailListPropertyConstants.SUBJECT_PREFIX, subject_prefix);
		return this.createList(MailAddress.parseSMTPStyle(listAddress), props);
	}
	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.entity.EntityMailListManagerMBean#addMember(org.jboss.mail.message.MailAddress, org.jboss.mail.message.MailAddress)
	 */
	public void addMember(MailAddress listAddress, MailAddress memberAddress) {
		try {
			InitialContext ctx = new InitialContext();
			MailListSBLocalHome mailListSBHome = (MailListSBLocalHome)ctx.lookup("jbossmail.EntityMailListSB");
			MailListSBLocal mailListSBLocal = mailListSBHome.create();
			mailListSBLocal.addMember(listAddress, memberAddress);
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (CreateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	/* (non-Javadoc)
	 * @see org.jboss.mail.maillist.entity.EntityMailListManagerMBean#addMemeber(java.lang.String, java.lang.String)
	 */
	public void addMemeber(String listAddress, String memberAddress) {
		MailAddress list = MailAddress.parseSMTPStyle(listAddress);
		MailAddress memeber = MailAddress.parseSMTPStyle(memberAddress);
		this.addMember(list,memeber);
	}

}
