/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist.entity.bean;

import java.util.Set;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.RemoveException;

import org.jboss.mail.maillist.MailListProperties;
import org.jboss.mail.maillist.MailListPropertyConstants;
import org.jboss.mail.maillist.entity.EntityMailList;
import org.jboss.mail.message.MailAddress;

/**
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
 *
 */
public abstract class MailListBean implements EntityBean, MailListPropertyConstants {
	
	/** Email address of the list.
	 */
	public abstract String getListAddress();
	public abstract void setListAddress(String listAddress);

	/** Whether the reply-to address for all messages sent to this 
	 * list is set to the list address.*/
	public abstract boolean getReplyToList();
	public abstract void setReplyToList(boolean replyToList);

	/** If set, this value is prepended to the subject line of all 
	 * messages sent to the list*/
	public abstract String getSubjectPrefix();
	public abstract void setSubjectPrefix(String prefix);
	
	/** If a subjectprefix is set, this value determines whether the 
	 * prefix is bracketed before being prepended to the subject line.*/
	public abstract boolean getPrefixAutoBracketed();
	public abstract void setPrefixAutoBracketed(boolean prefix);
   
   /** Whether attachments are allowed in messages sent to this list. 
	* Defaults to false*/
	public abstract boolean getAttachmentAllowed();
	public abstract void setAttachmentAllowed(boolean prefix);

	/** Whether attachments are allowed in messages sent to this list. 
	 * Defaults to false*/
	public abstract boolean getMembersOnly();
	public abstract void setMembersOnly(boolean membersOnly);
	
	/**CMR field for members in MailListMembers
	 */
	public abstract Set getMembers();
	public abstract void setMembers(Set members);

	public void ejbActivate() throws EJBException{}
	public void ejbLoad() throws EJBException{}
	public void ejbPassivate() throws EJBException{}
	public void ejbRemove()	throws RemoveException, EJBException{}
	public void ejbStore() throws EJBException{}
	public void setEntityContext(EntityContext ctx)	throws EJBException{}
	public void unsetEntityContext() throws EJBException{}


	public String ejbCreate(String listAddress, MailListProperties properties)throws CreateException{									
		setListAddress(listAddress);
		setProperties(properties);
		return null;
	}

	public void ejbPostCreate(String listAddress, MailListProperties properties)throws CreateException{
											
	}
	
	public EntityMailList getMailList(){
		MailAddress listAddress = MailAddress.parseSMTPStyle(getListAddress());
		
		MailListProperties props = new MailListProperties();
		props.setProperty(MailListProperties.REPLY_TO_LIST, new Boolean(getReplyToList()));
		props.setProperty(MailListProperties.SUBJECT_PREFIX, getSubjectPrefix());
		props.setProperty(MailListProperties.PREFIX_AUTO_BRACKETED, new Boolean(getPrefixAutoBracketed()));
		props.setProperty(MailListProperties.ATTACHMENT_ALLOWED, new Boolean(getAttachmentAllowed()));
		props.setProperty(MailListProperties.MEMBERS_ONLY, new Boolean(getMembersOnly()));
		
		EntityMailList mailList = new EntityMailList(listAddress, props);
		return mailList;
	}

	public void setProperties(MailListProperties properties){
		setReplyToList(properties.getPropertyBool(REPLY_TO_LIST).booleanValue());
		setSubjectPrefix(properties.getProperty(SUBJECT_PREFIX));
		setPrefixAutoBracketed(properties.getPropertyBool(PREFIX_AUTO_BRACKETED).booleanValue());
		setAttachmentAllowed(properties.getPropertyBool(ATTACHMENT_ALLOWED).booleanValue());
		setMembersOnly(properties.getPropertyBool(MEMBERS_ONLY).booleanValue());		
	}

	public void addMember(MailListMemberLocal mailListMember){
		Set members = getMembers();
		members.add(mailListMember);
	}

	public void removeMember(MailListMemberLocal mailListMember)throws RemoveException{
		Set members = getMembers();
		members.remove(mailListMember);
		mailListMember.remove();
	}


}
