/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist.entity.bean;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.EJBException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;
import org.jboss.mail.maillist.MailList;
import org.jboss.mail.maillist.MailListProperties;
import org.jboss.mail.maillist.MailListPropertyConstants;
import org.jboss.mail.maillist.entity.EntityMailList;
import org.jboss.mail.message.MailAddress;

/**
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
 *
 */
public class MailListSBBean implements SessionBean, MailListPropertyConstants {
	final static Logger logger = Logger.getLogger(MailListSBBean.class);
	 
	MailListLocalHome mailListLocalHome;
	MailListMemberLocalHome mailListMemberLocalHome;
	
	public void ejbCreate()throws CreateException{}	
	public void ejbActivate() throws EJBException{}
	public void ejbPassivate() throws EJBException{}
	public void ejbRemove() throws EJBException{}
	
	public void setSessionContext(SessionContext ctx) throws EJBException{
		try {
			InitialContext ic = new InitialContext();
				
			if (mailListLocalHome == null){
				mailListLocalHome = (MailListLocalHome)ic.lookup("jbossmail.EntityMailList");
			}
			if (mailListMemberLocalHome == null){
				mailListMemberLocalHome = (MailListMemberLocalHome)ic.lookup("jbossmail.EntityMailListMember");
			}
		} catch (NamingException e) {
			throw new EJBException(e);
		}
	}
	
	public boolean createList(MailAddress listAddress, MailListProperties properties){
		try {
			mailListLocalHome.create(getRawAddress(listAddress), properties);
			return true;
		} catch (CreateException e) {
			logger.error("Could not create list " + listAddress, e);
		}
		
		return false;
	}
	
	public void deleteList(MailAddress listAddress){
		try {
			MailListLocal mailList = mailListLocalHome.findByPrimaryKey(getRawAddress(listAddress));
			mailList.remove();
		} catch (FinderException e) {
			logger.warn("Could not find list we want to remove:" + listAddress);
		} catch (RemoveException e) {
			logger.error("Could not remove list:" + listAddress, e);
		}
	}
	
	public MailList findList(MailAddress listAddress){
		try {
			MailListLocal mailListLcl = mailListLocalHome.findByPrimaryKey(getRawAddress(listAddress));
			EntityMailList mailList = mailListLcl.getMailList();
			return mailList;
			
		} catch (FinderException e) {
			//Ignore and just return null
		}
		return null;
	}

	public void addMember(MailAddress listAddress, MailAddress address) {
		try {
			//Find the list
			String rawListAddress = getRawAddress(listAddress);
			MailListLocal mailListLcl = mailListLocalHome.findByPrimaryKey(rawListAddress);
			
			//Create the member
			MailListMemberPK pk = new MailListMemberPK(
										rawListAddress,
										getRawAddress(address));

			MailListMemberLocal mailListMember
							= mailListMemberLocalHome.create(
										rawListAddress,
										getRawAddress(address));

			//Add the member to the list
			mailListLcl.addMember(mailListMember);

			
		} catch (FinderException e) {
			logger.warn("Could not find list we want to add member to:" + listAddress);
		}catch(DuplicateKeyException e){
			//Let's just ignore it, there already			
		} catch (CreateException e) {
			logger.error("Could create member:" + address + " to be added to list:" + listAddress);
		}
	}

	public boolean isMember(MailAddress listAddress, MailAddress address) {	
		try {
			MailListMemberPK pk = new MailListMemberPK(
										getRawAddress(listAddress),
										getRawAddress(address));
			
			MailListMemberLocal mailListMember
							= mailListMemberLocalHome.findByPrimaryKey(pk);
			return true;
		} catch (FinderException e) {
		}
		return false;
	}

	public MailAddress[] getMembers(MailAddress listAddress) {
		ArrayList addresses = new ArrayList();
		try {
			//Find the list
			String rawListAddress = getRawAddress(listAddress);
			MailListLocal mailListLcl = mailListLocalHome.findByPrimaryKey(rawListAddress);
			Collection members = mailListLcl.getMembers();
			
			//Create MailAddresses from the members
			for (Iterator it = members.iterator() ; it.hasNext() ; ){
				MailListMemberLocal member = (MailListMemberLocal)it.next();
				addresses.add(MailAddress.parseSMTPStyle(member.getMemberEmail()));
			}
		} catch (FinderException e) {
			logger.warn("Could not find list we want to add member to:" + listAddress);
		}
		
		return (MailAddress[])addresses.toArray(new MailAddress[addresses.size()]);
	}

	public void removeMember(MailAddress listAddress, MailAddress address) {
		try {
			//Find the list
			String rawListAddress = getRawAddress(listAddress);
			MailListLocal mailList = mailListLocalHome.findByPrimaryKey(rawListAddress);
			
			//Remove the member
			MailListMemberPK pk = new MailListMemberPK(
										getRawAddress(listAddress),
										getRawAddress(address));
			
			MailListMemberLocal mailListMember
							= mailListMemberLocalHome.findByPrimaryKey(pk);
							
			mailList.removeMember(mailListMember);				

		} catch (FinderException e) {
			logger.warn("Could not remove member from list. " 
							+ listAddress + "; member:" + address + " not found");
		} catch (RemoveException e) {
			logger.warn("Could not remove: " + address + " from list: "	+ listAddress);
		}
	}

	public void setProperties(MailAddress listAddress, MailListProperties properties) {
		try {
			//Find the list
			String rawListAddress = getRawAddress(listAddress);
			MailListLocal mailList = mailListLocalHome.findByPrimaryKey(rawListAddress);
			mailList.setProperties(properties);

		} catch (FinderException e) {
			logger.warn("Could not find list we want to add member to:" + listAddress);
		}
	}

	private String getRawAddress(MailAddress address){
		return address.getUser() + "@" + address.getDomain();
	}
	
}
