/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillist.memory;

import java.util.HashSet;

import org.jboss.mail.maillist.MailList;
import org.jboss.mail.maillist.MailListProperties;
import org.jboss.mail.message.MailAddress;

/** Memory implementation of a MailList
*
* @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
*/
public class MemoryMailList implements MailList {
	/** The address of the list */
	MailAddress listAddress;

	/** The properties for this list */
	MailListProperties properties;

	/** The addresses in this list */
	HashSet addresses = new HashSet();
	
	MemoryMailList(MailAddress listAddress, MailListProperties properties){
		 this.listAddress = listAddress;
		 this.properties = properties;
	}

	/** Gets the address of the list 
	 * @return The address if the list
	 */
	public MailAddress getListAddress() {
		return listAddress;
	}

	/** Adds an address to the list
	 * @param address Address to be added 
	 */
	public void addMember(MailAddress address) {
		addresses.add(address);
	}

	/** Checks if an address exists in the list
	 * @param address Address to be added
	 * @return true if is in list
	 */
	public boolean isMember(MailAddress address) {
		return addresses.contains(address);
	}

	/** Gets the members of the list 
	 * @return a list of the members of the list
	 */
	public MailAddress[] getMembers() {
		//Clone should avoid problems to do with length if any 
		//addresses are removed/added while we create array
		HashSet copy = (HashSet)addresses.clone();
		return (MailAddress[])copy.toArray(new MailAddress[copy.size()]);
	}

	/** Removes an address from the list 
	 * @param address Address to be removed
	 */
	public void removeMember(MailAddress address) {
		addresses.remove(address);
	}

	/** Gets the properties of the list - a clone should be returned
	 * @return a clone of the MailListProperties for the list
	 */
	public MailListProperties getProperties() {
		return (MailListProperties)properties.clone();
	}

	/** Sets the properties of the list
	 * @param properties properties to be set
	 */
	public void setProperties(MailListProperties properties) {
		//Not sure if need to clone here, but feels safer...
		this.properties = (MailListProperties)properties.clone();
	}

}
