/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillistener;

import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.jboss.mail.MailListener;
import org.jboss.mail.ServiceProxy;
import org.jboss.mail.message.Message;
import org.jboss.mail.util.JBMailJMXUtil;

/**
 * Sends a message via a passed into the constructor JMX service with a "send" function.
 *
 * @author Andrew C. Oliver
 */
public class JMXMailListenerProxy implements ServiceProxy, MailListener {
    ObjectName on;
    MBeanServer srv;

    /**
     * constructor used by the MailListener mbean to pass in its ObjectNAme instance 
     * @param objectname of the calling mail listener (for a callback)
     */    
    public JMXMailListenerProxy(ObjectName objectname) {
           srv = JBMailJMXUtil.locateJBoss();
           on = objectname;                    
    }

    /**
     * MIKEA: THIS ASSUMES THAT JMS ENSURES RELIABLE DELIVERY -- 
     * BUT THIS IS PROBABLY NOT THE CASE. 
     * THUS, ACTUALLY, BEFORE RETURNING,
     * WE NEED TO FORMALLY COMMIT THE MESSAGE AND PERSIST IT IN CASE OF 
     * SERVER CRASH (A LA RELIABILITY AND 
     * PERSISTENCE REQUIREMENTS OF RFC 821) 
     * 
     * ANDY: Actually by definition JMS ensures reliable delivery.
     *
     * @return the message containing what could not be handled by this listener, null if 
     * the whole message was consumed
     */
    public Message send(Message msg) {
        try {
            return (Message)srv.invoke(on,"send",new Object[] {msg},new String[]{Message.class.getName()});
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }

    }
}
