/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillistener;

import javax.management.ObjectName;

import org.jboss.mail.ServiceProxy;
import org.jboss.mail.ServiceProxyFactory;
import org.jboss.mail.message.Message;
import org.jboss.system.ServiceMBean;

/**
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan</a>
 *
 */
public interface NukesPosterJMSMailListenerMBean extends ServiceMBean, ServiceProxyFactory {
    /**
     * set the type of destination to queue or topic
     * @param string "queue" or "topic"
     */
    void setDestinationType(String string);
    /**
     * @param string queue or topic to post to (for domains in the onServerDomainsList)
     */
    void setDestination(String string);
    /**
     * @return "queue" or "topic" whether we post to a queue or a topic
     */
    String getDestinationType();

    /**
     * @return String name of the queue or topic we're posting to for domains in the onServerDomainsList
     */
    String getDestination();

    /**
     * @param string name of DomainGroup MBean
     */
	void setDomainGroup(ObjectName on);
    /**
     * @return name of DomainGroup MBean
     */
	ObjectName getDomainGroup();
    /*
     * non javadoc: see ServiceProxyFactory interface
     */
    ServiceProxy produceServiceProxy();

    void setNukesForumPoster(ObjectName name);
	
    ObjectName getNukesForumPoster();

    /**
     * actually send the message.  This is used by the JMXMailListenerProxy to ACTUALLY send the mail
     * This checks to see that the address is indeed a mail list and the puts it on the queue or topic.
     * @param msg mail to send (we'll have to cast it to Mail)
     * @return the message containing what could not be handled by this listener, null if 
     * the whole message was consumed
     */
    Message send(Message msg);

    
}
