/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.message;

import java.util.ArrayList;

import org.jboss.mail.smtp.SMTPConstants;

/**
 * Class representing a failed mail message. It resets the headers as required
 * for a bounce message.
 *
 * @author KKhan
 */
public class BouncedMail extends Mail {

	public BouncedMail (Mail mail, MailAddress[] failed){
		super(mail);
		//Set the envelope from address to be empty
		super.from = MailAddress.parseSMTPStyle("<>");
		super.to = new ArrayList();
		super.to.add(new EnvelopedAddress(mail.from, javax.mail.Message.RecipientType.TO));
		//super.ih = mail.ih;
		
		String[] hdr = super.ih.getHeader(StandardMailHeaders.SUBJECT);
		String subject = "";
		if (hdr.length > 0){
			super.ih.removeHeader(StandardMailHeaders.SUBJECT);
			subject = hdr[0];
		}
		super.ih.addHeader(StandardMailHeaders.SUBJECT, "Undeliverable: " + subject);
		
		//Set from and return-path headers to be postmaster 
		String postmaster = "<" + System.getProperty(SMTPConstants.POSTMASTER) + ">";
		super.ih.removeHeader(StandardMailHeaders.FROM);
		super.ih.addHeader(StandardMailHeaders.FROM, postmaster);		
		super.ih.removeHeader(StandardMailHeaders.RETURN_PATH);
		super.ih.addHeader(StandardMailHeaders.RETURN_PATH, postmaster);		
		
		//Get rid of CC and BCC headers and make sure there is only one recipient in
		//the to header
		super.ih.removeHeader(StandardMailHeaders.TO);
		super.ih.addHeader(StandardMailHeaders.TO, super.to.get(0).toString());
		super.ih.removeHeader(StandardMailHeaders.CC);
		super.ih.removeHeader(StandardMailHeaders.BCC);
		
		StringBuffer bodybuf = new StringBuffer("Your message could not be delivered to the following recipients:\r\n");
		for (int i = 0 ; i < failed.length ; i++){
			bodybuf.append(failed[i].toString());
			bodybuf.append("\r\n");
		}

		super.body = bodybuf.toString().getBytes();
	}
}
