package org.jboss.mail.message;

import javax.mail.Message;
import javax.mail.Message.RecipientType;

/**
 * Enveloped Address lets us set the visibility/type of the address
 */
public class EnvelopedAddress extends MailAddress {
    private Message.RecipientType type;
    private boolean local;
    private boolean rcpt;


    /**
     * Constructor creates an enveloped address from a regular one. If it is already an EnvelopedAddress 
     * it acts as a copy constructor.
     * @param orig address to envelope 
     */
    public EnvelopedAddress(MailAddress orig) {
        super(orig);
        if (orig instanceof EnvelopedAddress) {
            EnvelopedAddress maorig = (EnvelopedAddress)orig; 
            this.type = maorig.type;
            this.local = maorig.local;
        } 
    }
    
    /**
     * Constructor takes an existing address and assigns a recipient type
     * @param address to envelope
     * @param type of recipient (BCC, CC, TO)
     */
    public EnvelopedAddress(MailAddress address, RecipientType type) {
        this(address);
        this.type = type;
    }

    /**
     * Specify what type of recipient this is (TO, CC, BCC)
     * @param type of recipient
     */
    public void setType(Message.RecipientType type) {
        this.type = type;
    }
    
    /**
     * @return type of recipient (TO, CC, BCC)
     */
    public Message.RecipientType getType() {
        return this.type;
    }
    
    /**
     * specify whether this address is local or remote
     * @param local or remote status
     */
    public void setLocal(boolean local) {
        this.local = local;
    }
    
    /**
     * @return whether this address is local or not
     */
    public boolean getLocal() {
        return local;
    }
    

    /**
     * Is this a recipient of the mail (not just something to print on the header)
     * @return recipient status
     */
    public boolean getRcpt() {
        return rcpt;
    }

    /**
     * Mark this is the recipient of the mail
     * @param rcpt true if we are actually going to deliver it to that account
     */
    public void setRcpt(boolean rcpt) {
        this.rcpt = rcpt;
    }

}
