/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.message;

import javax.mail.MessagingException;

import org.jboss.mail.smtp.SMTPProtocolInstance;

/**
 * Interface for basic mail-header functionality.
 * User does not have to worry about RFC822 compliance.
 * @author Michael Krause 
 * @author Andrew C. Oliver <acoliver ot jboss dat org> - Javadoc comments only! :-(
 */
public interface MailHeaders{
    /**
     * load the headers from the input stream.  once this is done the input stream should be right at the body.
     * @param is input stream to read the headers off of
     * @param smtpProtocol SMTPProtocolInstance used for constructing the mail
     * @param from The sender of the mail
     * @param to The receiver addresses
     * @throws MessagingException if something goes wrong in parsing the headers
     */
    public void load(java.io.InputStream is, 
    		SMTPProtocolInstance smtpProtocol, MailAddress from, MailAddress[] to) throws MessagingException, LoopDetectedException;
    /**
     * get all headers matching the key.  There could be more than one but generally the array will have one element.
     * @param name of header
     * @return values of any matching headers. (the keys are not returned)
     */
    public java.lang.String[] getHeader(java.lang.String name);
    /**
     * Add a header/key pair
     * @param name of header
     * @param value of header
     */
    public void addHeader(java.lang.String name, java.lang.String value);
    
    /**
     * @return an iterator of all header lines
     */
    public java.util.Iterator getAllHeaderLines();
	/**
	 * @param string
	 */
	public void removeHeader(String string);
}
