/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.naming.InitialContext;
import javax.net.ssl.SSLSocketFactory;

import org.jboss.mail.ServiceProxy;
import org.jboss.mail.pop3.handlers.POP3Handlers;
import org.jboss.mail.util.JBMailJMXUtil;
import org.jboss.security.SecurityDomain;
import org.jboss.system.ServiceMBeanSupport;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * POP3Protocol implements the POP3 Protocol for Server.  POP3 is the
 * protocol used to retrieve mails by org.jboss.org.jboss.mail clients.
 *
 * @author Eric Daugherty
 */
public class POP3Protocol extends ServiceMBeanSupport implements POP3ProtocolMBean, POP3Constants
{
    Map handlers;
    private static final String SENDER = "SENDER";
    Map properties;

	/** The name of the security domain to use with server sockets that support SSL
	 */
	protected String sslDomain;

    /**
     * constructs an SMTPProtocol with no configuration
     */
    public POP3Protocol() {
        handlers = POP3Handlers.instance();
    }

    public String getName() {
        return "POP3";
    }

    // documented in super class

    /* (non-Javadoc)
     * @see org.jboss.mail.Protocol#getProperties()
     */
    //TODO: return properties as XML
    public Element getProperties() {
        return null;
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.SMTPProtocolMBean#setProperties(org.w3c.dom.Element)
     */
    public void setProperties(Element properties) {
        //TODO: Copied from SMTPProtocol.  Should be refactord into common code!
        NodeList nodes = properties.getElementsByTagName("property");
        Map pproperties = new HashMap(nodes.getLength());
        for (int k = 0; k < nodes.getLength(); k++) {
            String pname = null;
            Object pval = null;
            Node node = nodes.item(k);
            NodeList subnodes = node.getChildNodes();
            for (int x = 0; x < subnodes.getLength(); x++) {
                Node subnode = subnodes.item(x);
                if (subnode.getNodeName().equals("name")) {
                    pname = subnode.getFirstChild().getNodeValue();
                } else if (subnode.getNodeName().equals("value")) {
                    pval = subnode.getFirstChild().getNodeValue();
                } else if (subnode.getNodeName().equals("values")) {
                    NodeList values = subnode.getChildNodes();
                    pval = new ArrayList(values.getLength());
                    List plist = (List) pval;
                    for (int y = 0; y < values.getLength(); y++) {
                        Node value = values.item(y);
                        if (value.getNodeName() != null
                            && value.getNodeName().equals("value")) {
                            plist.add(value.getFirstChild().getNodeValue());
                        }
                    }
                } /* else {
                                throw new RuntimeException("property and password pairs are all that are needed/permitted "+subnode.getNodeName());
                            }*/
            }
            if (pname == null || pval == null) {
                throw new RuntimeException("users need passwords and passwords need users...");
            }

            setProperty(pproperties, pname, pval);
        }
        this.properties = pproperties;
    }

    /**
     * helper function to set properties into the map, typing them and all...
     * @param pproperties the map which we're building
     * @param pname the name of the property
     * @param pval the value to set
     */
    private void setProperty(Map pproperties, String pname, Object pval) {
        if (pname.equals(SERVER_NAME)) {
            String sval = (String) pval;
            pproperties.put(pname, sval);
        } else if (pname.equals(USER_REPOSITORY)) {
            try {
                ServiceProxy proxy = (ServiceProxy) JBMailJMXUtil.getService((String) pval);
                pproperties.put(pname, proxy);
            } catch (Exception e) {
                e.printStackTrace();
                throw new RuntimeException(e);
            }
        } else if (pname.equals(REQUIRE_CLIENT_CERT)) {
			String sval = (String) pval;
			pproperties.put(pname, new Boolean(sval));
		} else if (pname.equals(TLS_SUPPORT)) {
			String sval = (String) pval;
			pproperties.put(pname, new Boolean(sval));
		}
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.ServiceProxyFactory#produceServiceProxy()
     */
    public ServiceProxy produceServiceProxy() {
        // TODO Auto-generated method stub
        return new POP3ProtocolInstance(handlers, properties);
    }

	/** Set the JAAS security domain to be used with server sockets that support SSL
	 * @param domainName The JNDI name of the security domain used with server sockets that support SSL
	 */
	public void setSecurityDomain(String domainName) {
		try{
			log.debug("setSecurityDomain()");
			this.sslDomain = domainName;
		}
		catch(RuntimeException e){
			log.error("Error", e);
			throw e;
		}
	}

	/** Get the JAAS security domain to be used with server sockets that support SSL
	 * @return The JNDI name of the security domain used with server sockets that support SSL
	 */
	public String getSecurityDomain() {
		return sslDomain;
	}

	protected void startService() throws Exception {
		log.debug("startService()");
		setupSSLSocketFactory();
	}

	/** Set up the SSLSocketFactory to be used to create the wrapper SSL Socket when
	 * SSL/TLS is to be used
	 */
	private void setupSSLSocketFactory() throws RuntimeException{
		//TODO: Duplicated from SMTPProtocol, refactor to common code?
		SSLSocketFactory sslSocketFactory = null;
		if (sslDomain != null) {
			ClassLoader loader = Thread.currentThread().getContextClassLoader();

			try {
				///////////////////////////////////////////////////////////
				// Here we want to use an SSLSocketFactory that understands
				// security domains. org.jboss.security.ssl.DomainSocket
				// does but doesn't exist in 3.2.3. Another implementation
				// has been provided for 3.2.3 hence this "clutter"!
				//
				// Once 3.2.4 has taken over the alternative implementation
				// should be scrapped.

				try{
					//This only exists in 3.2.4 and not in 3.2.3
					Class sfClass = loader.loadClass("org.jboss.security.ssl.DomainSocketFactory");
					sslSocketFactory = (SSLSocketFactory)sfClass.newInstance();
				}
				catch(Exception e){
				   throw new RuntimeException("Could not create SSLSocketFactory", e);
				}

				Class sfClass = sslSocketFactory.getClass();
				InitialContext ctx = new InitialContext();
				SecurityDomain domain = (SecurityDomain) ctx.lookup(sslDomain);
				Class[] parameterTypes = { SecurityDomain.class };
				Method m = sfClass.getMethod("setSecurityDomain", parameterTypes);
				Object[] args = { domain };
				m.invoke(sslSocketFactory, args);

			}
			catch(NoSuchMethodException e){
				log.error("Socket factory does not support setSecurityDomain(SecurityDomain)");
				throw new RuntimeException("Could not set up security for TLS", e);
			}
			catch (Exception e) {
				log.error("Failed to setSecurityDomain=" + sslDomain + " on socket factory", e);
				throw new RuntimeException("Could not set up security for TLS", e);
			}
		}
		else{
			//If no security domain has been created, it makes no sense to
			//have enabled tls and/or set tls to required
			Boolean prop = (Boolean)properties.get(TLS_SUPPORT);
			if (prop != null && prop.booleanValue()){
				throw new RuntimeException("tls-enabled was set to true, but no SecurityDomain was specified");
			}

			/*prop = (Boolean)properties.get(POP3Constants.REQUIRE_STARTTLS);
			if (prop != null && prop.booleanValue()){
				throw new RuntimeException("requireSTARTTLS was set to true, but no SecurityDomain was specified");
			}*/
		}

		properties.put(SSL_SOCKET_FACTORY, sslSocketFactory);
	}

}

