/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3;

import java.io.OutputStream;

import org.jboss.mail.AbstractResponse;
import org.jboss.mail.Response;
import org.jboss.mail.Protocol;
import org.jboss.mail.Request;

public class POP3Response extends AbstractResponse implements Response {

    /**
     * POP3Response constructor which creates an instance which is not terminal to the conversation
     *
     * @param request which generated this response
     * @param out stream which the response will be serialized to
     * @param protocol instance which is to be used
     */
    public POP3Response(
        Request request,
        OutputStream out,
        Protocol protocol) {
        this(request, out, protocol, false);
    }

    /**
     * POP3Response constructor which creates an instance which may or may not be terminal
     *
     * @param request which generated this response
     * @param out stream which the response will be serialized to
     * @param protocol instance which is to be used
     * @param finish whether or not this response terminates the conversation loop
     */
    public POP3Response(
        Request request,
        OutputStream out,
        Protocol protocol,
        boolean finish) {
        this.request = request;
        this.out = out;
        this.protocol = protocol;
        this.finish = finish;
    }
}
