/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3ProtocolInstance;

/**
 * Provides a base class for all POP3 CmdXXX classes.
 *
 * @author Eric Daugherty
 */
public class AbstractCommand implements POP3UserMessages {

    /**
     * Verifies that the number of arguments matches the expected
     * size.  If the size does not match, the proper message is
     * written to the client and false is returned.  Otherwise, true
     * is returned.
     * @param arguments the arguments to verify.
     * @param expectedCount the number of arguments expected
     * @return true if the size of the array matches the expectedCount
     */
    boolean verifyArgumentCount( String[] arguments, PrintWriter writer, int expectedCount ) {

        if( arguments.length < expectedCount) {
            writer.println( MESSAGE_ARGUMENT_MISSING );
            return false;
        }
        else if ( arguments.length > expectedCount ) {
        	//KAB: Check used to be: ( arguments.length > 1 )
        	//but POP3 commands can have more than one argument
        	writer.println( MESSAGE_TOO_MANY_ARGUMENTS );
            return false;
        }

        return true;
    }

    /**
     * Verifies that the current protocol state matches the
     * expected state for this command.  Writes error to users and
     * returns false if states do not match.
     *
     * @param expectedState the state required for this command to be valid.
     * @return true if the expectedState matches the current state.
     */
    boolean verifyState( POP3ProtocolInstance protocol, PrintWriter writer, int expectedState ) {

        // Verify the protocol is in the right state.
        if( protocol.getState() != expectedState )
        {
            writer.println( MESSAGE_INVALID_STATE );
            return false;
        }

        return true;
    }
}
