/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.apache.log4j.Logger;

/**
 * APOP alternative authentication mechanism.
 *
 * @author Eric Daugherty
 */
public class CmdAPOP extends AbstractCommand implements POP3Handler {

    public final static String COMMAND = "APOP";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdAPOP.class );

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug( "APOP command handler called") ;
        POP3Response response = new POP3Response( request, out, protocol );
        PrintWriter writer = response.getWriter();

        // Verify the protocol is in the right state.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_AUTHORIZATION ) )
        {
            //TODO Implement APOP Command
            writer.println( "-ERR Command Not Implemented" );
        }

        writer.flush();
        return response;
    }
}
