/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.apache.log4j.Logger;
import org.jboss.mail.pop3.POP3Constants;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3Response;

/**
 * Implements the CAPA command for POP3 as specified in RFC 2449
 * 
 * @author Kabir Khan
 */
public class CmdCAPA extends AbstractCommand implements POP3Handler, POP3Constants  {

	public final static String COMMAND = "CAPA";

	/** Log4j logger */
	private static final Logger log = Logger.getLogger( CmdCAPA.class );
	/* (non-Javadoc)
	 * @see org.jboss.mail.pop3.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.pop3.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
	 */
	public POP3Response handleRequest(
		OutputStream out,
		POP3Request request,
		POP3ProtocolInstance protocol)
		throws IOException {

			log.debug("CAPA command handler called");
			POP3ProtocolInstance popProtocol = (POP3ProtocolInstance)protocol; 
			POP3Response response = new POP3Response(request, out, protocol);
			PrintWriter writer = response.getWriter();
			int state = popProtocol.getState();
			if ( state != POP3ProtocolInstance.STATE_AUTHORIZATION
				&& state != POP3ProtocolInstance.STATE_TRANSACTION){
				writer.println( MESSAGE_INVALID_STATE );
			}else if(verifyArgumentCount( request.getArguments(), writer, 0 )){
				writer.println( MESSAGE_OK );
				//We have implemented the TOP, USER and UIDL commands, specified in 
				//the "initial capabilities" section of rfc 2449
				if (state == POP3ProtocolInstance.STATE_AUTHORIZATION){
					log.debug("auth state");
					writer.println(CmdUSER.COMMAND);
					
					if (protocol.getPropertyBool(TLS_SUPPORT)){
						writer.println(CmdSTLS.COMMAND);	
					}
				}
				else if (state == POP3ProtocolInstance.STATE_TRANSACTION){
					log.debug("tx state");
					writer.println(CmdTOP.COMMAND);
					writer.println(CmdUIDL.COMMAND);
				}
				
				writer.println(".");
			}
			writer.flush();
			return response;

	}

}
