/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.mailbox.Folder;
import org.jboss.mail.mailbox.MailboxMessage;
import org.apache.log4j.Logger;

/**
 * DELE marks a message for deletion.
 *
 * @author Eric Daugherty
 */
public class CmdDELE extends AbstractCommand implements POP3Handler  {

    public final static String COMMAND = "DELE";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdDELE.class );

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug( "DELE command handler called" );
        POP3Response response = new POP3Response(request, out, protocol);
        PrintWriter writer = response.getWriter();

        // Get the arguments.
        String[] arguments = request.getArguments();

        // Verify the current state and the arguement size.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_TRANSACTION ) &&
            verifyArgumentCount( arguments, writer, 1 ) )
        {
            Folder mailbox = protocol.getMailbox();
            MailboxMessage message = mailbox.getMessage( arguments[0] );
            if( message == null || message.isDeleted() )
            {
                writer.println( MESSAGE_INVALID_MESSAGE_ID );
            }
            else
            {
                message.setDeleted( true );
                writer.println( MESSAGE_OK );
            }
        }

        writer.flush();
        return response;
    }
}
