/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.apache.log4j.Logger;

/**
 * NOOP performs no action.  Generates
 * a positive response to the client.
 *
 * @author Eric Daugherty
 */
public class CmdNOOP extends AbstractCommand implements POP3Handler, POP3UserMessages {

    public final static String COMMAND = "NOOP";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdNOOP.class );

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug("NOOP command handler called");
        POP3Response response = new POP3Response(request, out, protocol );
        PrintWriter writer = response.getWriter();

        // Verify the protocol is in the right state.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_TRANSACTION ) )
        {
            // NOOP command just returns a positive (+OK) response
            writer.println( MESSAGE_OK );
        }

        writer.flush();
        return response;
    }
}
