/*
* JBoss, the OpenSource J2EE webOS
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.mailbox.Mailbox;
import org.jboss.mail.mailbox.MailboxManager;
import org.jboss.mail.mailbox.Folder;
import org.jboss.mail.userrepository.UserRepository;

import org.jboss.logging.Logger;

/**
 * PASS accepts a password as part
 * of the authentication process.
 *
 * @author Eric Daugherty
 */
public class CmdPASS extends AbstractCommand implements POP3Handler {

    public final static String COMMAND = "PASS";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdPASS.class );

    /* (non-Javadoc)
    * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
    */
    public POP3Response handleRequest(
            OutputStream out,
            POP3Request request,
            POP3ProtocolInstance protocol)
            throws IOException {

        log.debug("PASS command handler called");
        POP3Response response = new POP3Response(request, out, protocol);
        PrintWriter writer = response.getWriter();

        // Get the arguments and verify the size.
        String[] arguments = request.getArguments();

        // Verify the current state and the arguement size.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_AUTHORIZATION ) &&
            verifyArgumentCount( arguments, writer, 1 ) ) {

            String username = protocol.getUsername();
            String password = arguments[0];
            password.trim();

            // Verify the USER command has been accepted.
            if( username == null )
            {
                writer.println( MESSAGE_USER_REQUIRED_BEFORE_PASS );
            }
            // Verify password exists
            else if( password.length() == 0 )
            {
                writer.println( MESSAGE_PASS_INVALID );
            }
            // Verify the password.
            else if( password.length() > 0 ) {

                // For now, just accept all passwords.
                // TODO: Add Password check.
               if (((UserRepository) protocol.getProperty(USER_REPOSITORY)).test(
                username,
                password)== false) {
                  writer.println( "-ERR Authentication failed");
                  protocol.setState(USER,null);
                } else {
                  //protocol.setState(USER, user);

                    // If the password is valid, load the mailbox.
                    try {
                        MailboxManager mailboxManager = Mailbox.getMailboxManager();
                        Folder mailbox = mailboxManager.getMailbox( username );

                        // If the mailbox is null here, something is wrong since we
                        // already authenticated the user!
                        if( mailbox == null ) {
                            writer.println( MESSAGE_SERVICE_UNAVAILABLE );
                        }
                        // Attempt to lock the mailbox.
                        else if( mailbox.lock() )
                        {
                            protocol.setMailbox( mailbox);
                            protocol.setState( POP3ProtocolInstance.STATE_TRANSACTION );
                            writer.println( MESSAGE_OK );
                        }
                        else
                        {
                            writer.println( MESSAGE_MAILBOX_LOCKED );
                        }
                    }
                    catch (Exception e) {
                        log.error( "Error getting MailboxMBean: " + e, e );
                        writer.println( MESSAGE_SERVICE_UNAVAILABLE );
                    }

                //TODO: This looks like the code used by SMTPAuth but I think
                // we have the wrong protocol reference.  need to look into how
                // to get the UserRepository reference.
                /*
                UserRepository userRepository = protocol.getProperty(USER_REPOSITORY);
                if( userRepository.test( username, password ) ) {
                protocol.setState( POP3ProtocolInstance.STATE_TRANSACTION );
                writer.println( MESSAGE_OK );
                }
                else {
                protocol.setUsername( null );
                writer.println( MESSAGE_AUTH_FAILED );
                }
                */
                //writer.println( "-ERR PASS not implemented.");
                }
            }

        }

        writer.flush();
        return response;
    }
}
