/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.OutputStream;
import java.io.IOException;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3Constants;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.mailbox.Folder;
import org.apache.log4j.Logger;

/**
 * QUIT Ends the session.
 *
 * @author Eric Daugherty
 */
public class CmdQUIT implements POP3Handler, POP3UserMessages {

    public final static String COMMAND = "QUIT";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdQUIT.class );

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug("QUIT command handler called");
        POP3Response response = new POP3Response(request, out, protocol, true);
        PrintWriter writer = response.getWriter();

        // If the user is in the TRANSACTION state, commit any changes.
        if( protocol.getState() == POP3ProtocolInstance.STATE_TRANSACTION )
        {
            protocol.setState( POP3ProtocolInstance.STATE_UPDATE );
            Folder mailbox = protocol.getMailbox();
            if( mailbox != null )
            {
                // Commit the changes.  If the commit fails, respond
                // with an error message and return.
                if( !mailbox.commit() )
                {
                    writer.println( MESSAGE_COMMIT_FAILED );
                    writer.flush();
                    return response;
                }
            }
        }

        // Sign off
        writer.println(
            "+OK "
            + protocol.getProperty(POP3Constants.SERVER_NAME)
            + " Service closing transmission channel");

        writer.flush();
        return response;
    }
}
