/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.mailbox.MailboxMessage;
import org.jboss.mail.mailbox.Folder;
import org.apache.log4j.Logger;

/**
 * RSET unmarks any messages that
 * have been marked for deletion.
 *
 * @author Eric Daugherty
 */
public class CmdRSET extends AbstractCommand implements POP3Handler {

    public final static String COMMAND = "RSET";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdRSET.class );

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug( "RSET command handler called" );
        POP3Response response = new POP3Response(request, out, protocol);
        PrintWriter writer = response.getWriter();

        // Verify the protocol is in the right state.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_TRANSACTION ) )
        {
            Folder mailbox = protocol.getMailbox();
            MailboxMessage[] messages = mailbox.getMessages();

            // Clear the deleted flag on all messages.
            for( int index = 0; index < messages.length; index++ )
            {
                messages[index].setDeleted( false );
            }
            writer.println( MESSAGE_OK );
        }

        writer.flush();
        return response;
    }
}
