/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.mailbox.Folder;
import org.jboss.mail.mailbox.MailboxMessage;
import org.apache.log4j.Logger;

/**
 * STAT generates a one line summary of
 * the user's mailbox.
 *
 * @author Eric Daugherty
 */
public class CmdSTAT extends AbstractCommand implements POP3Handler, POP3UserMessages {

    public final static String COMMAND = "STAT";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdSTAT.class );


    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
     */
    public POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException {

        log.debug( "STAT command handler called" );
        POP3Response response = new POP3Response(request, out, protocol);
        PrintWriter writer = response.getWriter();

        // Verify the protocol is in the right state.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_TRANSACTION ) )
        {
            Folder mailbox = protocol.getMailbox();
            MailboxMessage[] messages = mailbox.getMessages();

            long totalCount = 0;
            long totalSize = 0;
            // Calculate the total size.
            for( int index = 0; index < messages.length; index++ )
            {
                if( !messages[index].isDeleted() )
                {
                    totalCount++;
                    totalSize += messages[index].getSize();
                }
            }

            // Output the response
            writer.println( "+OK " + totalCount + " " + totalSize );
        }

        writer.flush();
        return response;
    }
}
