/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.security.cert.Certificate;

import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

import org.apache.log4j.Logger;
import org.jboss.mail.ServerThread;
import org.jboss.mail.pop3.POP3Constants;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3Response;

/**
 * Implements the STARTTLS-Command for POP3
 * 
 * @author Kabir Khan
 */
public class CmdSTLS extends AbstractCommand implements POP3Handler, POP3UserMessages, POP3Constants {

	public final static String COMMAND = "STLS";

	/** Log4j logger */
	private static final Logger log = Logger.getLogger( CmdSTLS.class );
	/* (non-Javadoc)
	 * @see org.jboss.mail.pop3.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.pop3.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
	 */
	public POP3Response handleRequest(
		OutputStream out,
		POP3Request request,
		POP3ProtocolInstance protocol)
		throws IOException {

			log.debug("STLS command handler called");
			POP3ProtocolInstance popProtocol = (POP3ProtocolInstance)protocol; 
			POP3Response response = new POP3Response(request, out, protocol);
			PrintWriter writer = response.getWriter();
			
			if (popProtocol.getState() != POP3ProtocolInstance.STATE_AUTHORIZATION) {
				writer.println( MESSAGE_INVALID_STATE );
			}else if(verifyArgumentCount( request.getArguments(), writer, 0 )){
				writer.println( MESSAGE_OK );
				java.security.Security.addProvider( new com.sun.net.ssl.internal.ssl.Provider());
				//     System.setProperty("javax.net.debug", "ssl,handshake");
				SSLSocketFactory factory = 
					(SSLSocketFactory)popProtocol.getProperty(SSL_SOCKET_FACTORY);
				Socket oldSocket = popProtocol.getServerThread().getSocket();
				final SSLSocket newSocket = 
					(SSLSocket)factory.createSocket(oldSocket,
													oldSocket.getLocalAddress().getHostName(),
													oldSocket.getLocalPort(),
													true);
				newSocket.setNeedClientAuth(protocol.getPropertyBool(REQUIRE_CLIENT_CERT));
				newSocket.setUseClientMode(false);
				final POP3ProtocolInstance tempProtocol = (POP3ProtocolInstance)protocol;
				newSocket.addHandshakeCompletedListener(new HandshakeCompletedListener(){
					public void handshakeCompleted(HandshakeCompletedEvent event){
						log.debug("HS completed");
						tempProtocol.setSecure(true);
                    
						//Reset the state but carry over the addresses connected as this is needed
						//by MailHeadersImpl
						Object localAddr = tempProtocol.getState(ServerThread.STATE_LOCAL_ADDRESS);
						Object remoteAddr = tempProtocol.getState(ServerThread.STATE_CLIENT_ADDRESS);
						tempProtocol.resetState();
						tempProtocol.setState(ServerThread.STATE_LOCAL_ADDRESS, localAddr);
						tempProtocol.setState(ServerThread.STATE_CLIENT_ADDRESS, remoteAddr);
                    
                    
						log.debug("State reset");
						Certificate certs[] = null;
						try{
							certs = newSocket.getSession().getPeerCertificates();
							int j;
							for(j=certs.length -1; j >= 0; j--){
								log.debug(certs[j]);
							}
							//TODO: Put some info about the Certificate in Mail-Header
						}catch(SSLPeerUnverifiedException e){
							e.printStackTrace();
						}
					}
				});
            
				ServerThread serverThread = ((POP3ProtocolInstance)protocol).getServerThread(); 
				serverThread.setSocket(newSocket);
			}
			writer.flush();
			return response;

	}

}
