/*
* JBoss, the OpenSource J2EE webOS
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.pop3.POP3Response;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.apache.log4j.Logger;

/**
 * USER accepts a username as part
 * of the authentication process.
 *
 * @author Eric Daugherty
 */
public class CmdUSER extends AbstractCommand implements POP3Handler, POP3UserMessages {

    public final static String COMMAND = "USER";

    /** Log4j logger */
    private static final Logger log = Logger.getLogger( CmdUSER.class );

    /* (non-Javadoc)
    * @see org.jboss.mail.smtp.handlers.POP3Handler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.POP3Request, org.jboss.mail.pop3.POP3ProtocolInstance)
    */
    public POP3Response handleRequest(
            OutputStream out,
            POP3Request request,
            POP3ProtocolInstance protocol)
            throws IOException {

        log.debug( "USER command handler called" );
        POP3Response response = new POP3Response(request, out, protocol);
        PrintWriter writer = response.getWriter();

        // Get the arguments
        String[] arguments = request.getArguments();

        // Verify the current state and the arguement size.
        if( verifyState( protocol, writer, POP3ProtocolInstance.STATE_AUTHORIZATION ) &&
            verifyArgumentCount( arguments, writer, 1 ) )
        {
            // Verify a user has not already been specified
            if( protocol.getUsername() != null ) {
                writer.println( MESSAGE_USER_ALREADY_SPECIFIED );
                return response;
            }

            // Set the user into the protocol if valid.
            String username = arguments[0];
            username.trim().toLowerCase();
            if( username.length() > 0 ) {
                protocol.setUsername( username );
                log.warn("pop username set to "+username);
                writer.println( MESSAGE_OK );
            }
            else {
                writer.println( MESSAGE_USER_INVALID );
            }
        }

        writer.flush();
        return response;
    }
}
