/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.io.IOException;
import java.io.OutputStream;

import org.jboss.mail.pop3.POP3ProtocolInstance;
import org.jboss.mail.pop3.POP3Request;
import org.jboss.mail.pop3.POP3Response;

/**
 * POP3Handler interface defines the method to which requests are passed.  It is called by the
 * POP3Protocol in order to actually handle the request.  New handlers can be added (presently
 * statically, though extensibility is feasible) easily to the POP3Handlers class.
 *
 * @author Eric Daugherty
 */
public interface POP3Handler {

    /**
     * Handle the processing of the request
     * @param out outputstream in which messages and responses are written
     * @param request which is to be processed
     * @param protocol instance which delegated the request
     * @return Response providing information about the processing of the request.
     * @throws java.io.IOException
     */
    POP3Response handleRequest(
        OutputStream out,
        POP3Request request,
        POP3ProtocolInstance protocol)
        throws IOException;
}
