/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.pop3.handlers;

import java.util.Map;
import java.util.HashMap;

/**
 * This class forms a handler collection which allows the caller to receive a statically held instance
 * of the handler Map.  Handlers are singleton instances in nature delegating all session information
 * up to the Protocol handler.  Handlers are mapped in such a way that the key value from their "COMMAND"
 * member determines their use.
 *
 * @author Eric Daugherty
 */
public class POP3Handlers {

    private static Map handlers;

    /**
     * return the map of handlers
     * @return Map containing the handlers mapped to their keys
     */
    public static Map instance() {
        if (handlers == null) {
            constructHandlers();
        }
        return handlers;
    }

    /**
     * static internal initializer which is called to construct the map of handlers in the event the
     * mapping is null.
     */
    private static synchronized void constructHandlers() {
        if (handlers == null) {
            handlers = new HashMap();

            handlers.put(CmdSTAT.COMMAND, new CmdSTAT());

            handlers.put(CmdLIST.COMMAND, new CmdLIST());

            handlers.put(CmdRETR.COMMAND, new CmdRETR());

            handlers.put(CmdDELE.COMMAND, new CmdDELE());

            handlers.put(CmdNOOP.COMMAND, new CmdNOOP());

            handlers.put(CmdRSET.COMMAND, new CmdRSET());

            handlers.put(CmdQUIT.COMMAND, new CmdQUIT());

            handlers.put(CmdTOP.COMMAND, new CmdTOP());

            handlers.put(CmdUIDL.COMMAND, new CmdUIDL());

            handlers.put(CmdUSER.COMMAND, new CmdUSER());

            handlers.put(CmdPASS.COMMAND, new CmdPASS());

            handlers.put(CmdAPOP.COMMAND, new CmdAPOP());

			handlers.put(CmdSTLS.COMMAND, new CmdSTLS());
			
			handlers.put(CmdCAPA.COMMAND, new CmdCAPA());
        }
    }
}
