/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp;

/**
 * SMTP Constants contains constants for use throughout, most classes will just implement this interface
 * and use them directly.
 * 
 * @author Andrew C. Oliver
 * @copyright SuperLink Software, Inc. 2003 All rights reserved unless otherwise granted.
 */
public interface SMTPConstants {
	//properties
	/**
	 * Server Name for use in Protocol.getProperty
	 */
	public final static String SERVER_NAME = "servername";
	
	/**
	 * Is authentication requred... for use in Protocol.getProperty
	 */	
	public final static String AUTH_REQUIRED = "auth-required";

	/**
	 * Server Name for use in Protocol.getProperty
	 */
	public final static String STATUS = "STATUS";
	
	/**
	 * allowed methods for authentication in Protocol.getProperty
	 */
	public final static String AUTH_METHODS = "auth-methods";
	
	/**
	 * returns the user repository for use in authentication/authorization use with Protocol.getProperty
	 */
	public final static String USER_REPOSITORY = "user-repository";
	
	/**
	 * returns whether or not we will verify identity, use with Protocol.getProperty
	 */
	public final static String VERIFY_IDENTITY = "verify-identity";
	
	/**
	 * returns the set of Mail listeners for sending messages, use with Protocol.getProperty
	 */
	public final static String MAIL_LISTENERS = "mail-listeners";
	
	/**
	 * returns the maximum allowed message size when passed to Protocol.getProperty
	 */
	public final static String MAX_MSG_SIZE = "max-message-size";
	
	/**
	 * determines if the use of the STARTTLS-Extension is required
	 */
    public final static String REQUIRE_STARTTLS = "requireSTARTTLS";
	/**
	 * determnines id the client needs to pass a certificate during SSL-Handshake after STARTTLS
	 */
    public final static String REQUIRE_CLIENT_CERT = "requireClientCert";
    /**
     * enable or don't enable TLS (different from requireSTARTTLS as it determines whether we'll even allow it)
     */
    public static final String TLS_SUPPORT = "tls-enabled"; 
    /**
     * block size for messages 
     */
     public final static String BLOCK_SIZE = "block-size";
    
    /** 
     * The socket factory to be used to create sockets used for TLS
     */
    public static final String SSL_SOCKET_FACTORY = "SSL_SOCKET_FACTORY";

	/**
	 * The account used for the postmaster (also gets set in System properties)
	 */
	public static final String POSTMASTER = "postMaster";
	
	/**
	 * The maximum number of received headers allowed (to avoid looping). 
	 * If greater than this the message is not accepted. 
	 */
	public static final String MAX_RECEIVED_HEADERS = "maxReceivedHdrs";

	/**
	 * The number of received headers before we start inspecting the content looking for ourselves (to avoid looping)
	 */
	public static final String RECEIVED_HEADERS_THRESHOLD = "receivedHdrsThreshold";

	/**
	 * The number of times we can find ourselves in the received headers (to avoid looping)
	 * If greater than this the message is not accepted. 
	 */
	public static final String MAX_OWN_RECEIVED_HEADERS = "maxOwnReceivedHdrs";

	//states
	
	/**
	 * get the domain passed in the HELO or EHLO command from the Protocol.getState
	 */
	public final static String HELO_DOMAIN = "h";	
	
	/**
	 * get the user from the Protocol.getState
	 */
	public final static String USER = "USER";
	
	/**
	 * get the sender from the Protocol.getState
	 */
	public final static String SENDER = "SENDER";

	/**
	 * get the receipient list from the Protocol.getState
	 */
	public final static String RCPT_LIST = "RCPT_LIST";
	
	//authentication types
	/**
	 * for use in determining the authentication type (PLAIN digest) 
	 */
	public final static String AUTH_TYPE_PLAIN = "PLAIN";
	/**
	 * for use in determining the authentication type (LOGIN method)
	 */
	public final static String AUTH_TYPE_LOGIN = "LOGIN";
    /**
     * stores the message's size (CmdMAIL)
     */
    public static final String MSG_SIZE = null;

	//other
	/**
	 * list of SMTP Terminators for the filtering IO stream.
	 */
	public final static char[] SMTPTerminator = { '\r', '\n', '.', '\r', '\n' };

    public static final String MAIL_LISTENER = "mail-listener";

    /**
     * if STARTTLS is used, this is the name of the header in which the certificate
     * of the sender is put
     */
    public static final String MAIL_HEADER_CERT = "X-Sender-Certificate";

    public static final String DOMAIN_GROUP = "DOMAIN_GROUP";

}
