/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp;

import javax.management.ObjectName;

import org.jboss.mail.ServiceProxyFactory;
import org.jboss.system.ServiceMBean;
import org.w3c.dom.Element;

/**
 * SMTPProtocol is used to send mails.  We've implemented the Server side of it.  JavaMail implements 
 * ONLY the client side of it... 
 *
 * @author Andrew C. Oliver
 */
public interface SMTPProtocolMBean extends ServiceMBean, ServiceProxyFactory {

    /**
     * used by the XMBean stuff to set up our properties (see sample)
     * @param properties Element from JBoss
     */
    void setProperties(Element properties);
    /**
     * return the proeprties as a DOM object
     * @return Element containing our properties...
     */
    Element getProperties();

    /**
     * set the domain group that we depend on for VerifyIdentiy and other such properties
     * @param domainGroup
     */
    void setDomainGroup(ObjectName domainGroup);

    /**
     * @return the domain group that we depend on for VerifyIdentity.
     */
    ObjectName getDomainGroup();
    
	/** Set the JAAS security domain to be used with server sockets that support SSL
	 * @param domainName The JNDI name of the security domain used with server sockets that support SSL
	 */
	public void setSecurityDomain(String domainName);

	/** Get the JAAS security domain to be used with server sockets that support SSL
	 * @return The JNDI name of the security domain used with server sockets that support SSL
	 */
	public String getSecurityDomain();

}

