/*
* JBoss, the OpenSource J2EE webOS
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.mail.smtp;

import org.jboss.mail.Protocol;
import org.jboss.mail.Request;
 
import java.io.InputStream;

import java.util.Iterator;

/**
 * The specific SMTPRequest type extends the basic request with various SMTP-specific request properties
 * 
 * @author Andrew C. Oliver
 * @author Eric Daugherty
 */
public interface SMTPRequest extends Request {

    /**
     * SMTP Request command such as HELO, EHLO, QUIT, MAIL, etc.
     * @return command to be processed
     */
    public String getCommand();

    /**
     * arguments to the command
     * @see #getCommand()
     * @return arguments iterator
     */
    public Iterator arguments();

    /**
     * Returns an array of Strings that contains
     * the arguments for the current command.
     *
     * @return array of 0 or more strings.  Never null.
     */
    public String[] getArguments();

    /**
     * @return protocol instance which this request is attached to
     */
    public Protocol getProtocol();

    /**
     * @return input stream from which this request was parsed.
     */
    public InputStream getInputStream();
}
