/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp;

import java.io.OutputStream;

import org.jboss.mail.AbstractResponse;
import org.jboss.mail.Protocol;

/**
 * SMTPResponse is the SMTP-specific extension to the basic Response which adds SMTP properties and 
 * constructors.
 * 
 * @author Andrew C. Oliver
 * @copyright SuperLink Software, Inc. 2003 All rights reserved unless otherwise granted.
 */
public class SMTPResponse extends AbstractResponse
{

    /**
     * SMTPResponse constructor which creates an instance which is not terminal to the conversation
     * @param request which generated this response
     * @param out stream which the response will be serialized to
     * @param log the serverLog to log to (conversational logging)
     * @param protocol instance which is to be used
     */
    public SMTPResponse(
        SMTPRequest request,
        OutputStream out,
        Protocol protocol) {
        this(request, out, protocol, false);
    }

    /**
     * SMTPResponse constructor which creates an instance which may or may not be terminal
     * @param request which generated this response 
     * @param out stream which the response will be serialized to
     * @param log the serverLog to log to (conversational logging)
     * @param protocol instance which is to be used
     * @param finish whether or not this response terminates the conversation loop
     */
    public SMTPResponse(
        SMTPRequest request,
        OutputStream out,
        Protocol protocol,
        boolean finish) {
        this.request = request;
        this.out = out;
        this.protocol = protocol;
        this.finish = finish;
    }

}
