/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license. See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Iterator;

import org.jboss.logging.Logger;
import org.jboss.mail.Protocol;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPProtocolInstance;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;

/**
 * HELO and EHLO are basically equivilent. One signifies a server which has
 * extended SMTP capabilities. Primarily, authentication/etc. This is the
 * handshake which begins the session.
 * 
 * @author Andrew C. Oliver @copyright SuperLink Software, Inc. 2003 All rights
 *         reserved unless otherwise granted.
 */
public class CmdEHLO implements SMTPHandler, SMTPConstants {
    private static final Logger jblog = Logger.getLogger(CmdEHLO.class);

    public final static String COMMAND = "EHLO";

    public SMTPResponse handleRequest(
        OutputStream out,
        SMTPRequest request,
        Protocol protocol)
        throws IOException {
        jblog.debug("EHLO command handler called");
        SMTPProtocolInstance inst = (SMTPProtocolInstance) protocol;
        SMTPResponse response = new SMTPResponse(request, out, protocol);
        PrintWriter writer = response.getWriter();
        String remoteaddr = getRemoteAddr(request.arguments());
        if (remoteaddr != null && !remoteaddr.equals("")) {
            remoteaddr = remoteaddr.trim();
            writer.println(
                "250-"
                    + protocol.getProperty(SERVER_NAME)
                    + " Hello "
                    + remoteaddr
                    + " ("
                    + request.getRemoteAddr()
                    + ")");

            if (protocol.getPropertyBool(SMTPConstants.TLS_SUPPORT)) {
                writer.println("250-STARTTLS");
            }
            writer.println("250 " + protocol.getProperty(AUTH_METHODS));
            inst.setState(HELO_DOMAIN, remoteaddr);
        } else {
            writer.println("501 Domain address required: EHLO");
        }
        writer.flush();
        return response;
    }

    /**
	 * @param iter
	 *            containing a list or the argument lines (which SMTP doesn't
	 *            do anyhow)
	 * @return String containing the required remote address.
	 */
    private String getRemoteAddr(Iterator iter) {
        String remote = null;
        if (iter.hasNext() && iter != null) {
            remote = (String) iter.next();
        }
        return remote;
    }
}
