/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import org.jboss.logging.Logger;
import org.jboss.mail.Protocol;

import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.util.Map;
import java.util.HashMap;

/**
 * HELP command, recommended by rfc 821 
 * 
 * @author <a href='mailto:mikea@xoba.com'> mike andrews </a>
 */
public class CmdHELP implements SMTPHandler, SMTPConstants {

    private static final Logger jblog = Logger.getLogger(CmdHELO.class);

    private final static Map topics = new HashMap();

    static {
	topics.put("HELO",new String[] { "HELO <hostname>", "this command identifies the MTA to this server."});
	topics.put("EHLO",new String[] { "HELO <hostname>", "this command identifies the MTA to this server",
					 "requesting extended SMTP mode." });
	topics.put("MAIL",new String[] { "MAIL FROM:<sender>",
					 "identifies the mail sender." });
	topics.put("RCPT",new String[] { "RCPT TO:<recipient>", 
					 "identifies a mail recipient." } );
	topics.put("DATA",new String[] { "DATA",
					 "following text is collected as the message.",
					 "end with single dot on line followed by CRLF.",
					 "otherwise, escape leading dot using \\." } );
	topics.put("RSET",new String[] { "RSET", "resets the system."});
	topics.put("NOOP",new String[] { "NOOP", "does nothing."});
	topics.put("QUIT",new String[] { "QUIT", "terminate interaction with server."});
	topics.put("VRFY",new String[] { "VRFY <recipient>", "verify an address."});
    }
    
    public final static String COMMAND = "HELP";

    public SMTPResponse handleRequest(OutputStream out, SMTPRequest request, Protocol protocol) throws IOException {
	jblog.debug("HELP command handler called");
	SMTPResponse response = new SMTPResponse(request, out, protocol);
	PrintWriter writer = response.getWriter();
	String[] args = request.getArguments();
	if (args.length > 0) {
	    String topic = args[0].toUpperCase();
	    if (topics.containsKey(topic)) {
		Object text = topics.get(topic);
		if (text instanceof Object[]) {
		    Object[] lines = (Object[]) text;
		    // multiple line response, indent trailing lines
		    for (int i=0; i<lines.length; i++) {
			writer.println("214 " + (i > 0 ? "   " : "") + lines[i]);
		    }
		    writer.println("214 [end of help info]");
		} else {
		    // single-line response
		    writer.println("214 " + text);
		}
	    } else {
		writer.println("501 HELP topic \""+topic+"\" not recognized");
	    }
	} else {
	    writer.println("214 Topics:");
	    writer.println("214    HELO EHLO MAIL RCPT DATA");
	    writer.println("214    RSET NOOP QUIT HELP VRFY");
	    writer.println("214 for more information use \"HELP <topic>\"");
	    writer.println("214 to report bugs, contact jboss-mail@jboss.org");
	    writer.println("214 [end of help info]");
	}
	writer.flush();
	return response;
    }
}
