/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.mail.Protocol;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;
import org.jboss.logging.Logger;

/**
 * QUIT Ends the session
 * 
 * @author Andrew C. Oliver
 */
public class CmdQUIT implements SMTPHandler {

    private static final Logger jblog = Logger.getLogger(CmdQUIT.class);
    public final static String COMMAND = "QUIT";

    /*
     *  (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.SMTPHandler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.SMTPRequest, org.jboss.mail.Protocol)
     */
    public SMTPResponse handleRequest(
        OutputStream out,
        SMTPRequest request,
        Protocol protocol)
        throws IOException {
        jblog.debug("QUIT command handler called");
        SMTPResponse response = new SMTPResponse(request, out, protocol, true);
        PrintWriter writer = response.getWriter();
        writer.println(
            "221 "
                + protocol.getProperty("servername")
                + " closing connection");
        writer.flush();
        request.getInputStream().mark(2);
     //   request.getInputStream().read(new byte[2]);//hack - swallow the last carriage return
        return response;
    }
}
