/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.logging.Logger;
import org.jboss.mail.Protocol;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;

/**
 * The RSET command clears out the state of the transmission.  It should make things as if we were just
 * after HELO.
 * 
 * @author Kabir Khan
 * @author Andrew C. Oliver <acoliver ot jboss dat org> - javadoc only
 */
public class CmdRSET implements SMTPHandler, SMTPConstants
{
	public final static String COMMAND = "RSET";
	private static final Logger jblog = Logger.getLogger(CmdRSET.class);


	/* (non-Javadoc)
	 * @see org.jboss.mail.smtp.handlers.SMTPHandler#handleRequest(java.io.OutputStream, org.jboss.mail.smtp.SMTPRequest, org.jboss.mail.Protocol, org.jboss.mail.log.MailLog)
	 */
	public SMTPResponse handleRequest(
		OutputStream out,
		SMTPRequest request,
		Protocol protocol)
		throws IOException {
		jblog.debug("RSET command handler called");

		SMTPResponse response = new SMTPResponse(request, out, protocol);
		PrintWriter writer = response.getWriter();
		
		//Moan about arguments (should be none)
		String[] arguments = request.getArguments(); 
		if (arguments != null && arguments.length > 0){
			writer.println("501 Usage: RSET");
		}
		else{
			//should authentication be undone?  JAMES doesn't
			protocol.setState(SENDER, null);
			protocol.setState(RCPT_LIST, null);
			writer.println("250 state reset OK");
		}
        
		writer.flush();
		return response;
	}

}

