/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.security.cert.Certificate;

import javax.net.ssl.HandshakeCompletedEvent;
import javax.net.ssl.HandshakeCompletedListener;
import javax.net.ssl.SSLPeerUnverifiedException;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;

import org.jboss.logging.Logger;
import org.jboss.mail.Protocol;
import org.jboss.mail.ServerThread;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPProtocolInstance;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;


/**
 * Implements the STARTTLS-Command described in rfc 2487.
 * 
 * @author Michael Krause
 */
public class CmdSTARTTLS implements SMTPHandler, SMTPConstants {

    public final static String COMMAND = "STARTTLS";
    private static final Logger jblog = Logger.getLogger(CmdSTARTTLS.class);

    public SMTPResponse handleRequest(OutputStream out, SMTPRequest request, Protocol protocol) throws IOException {
        jblog.debug("STARTTLS command handler called");
        SMTPResponse response = new SMTPResponse(request, out, protocol);
        PrintWriter writer = response.getWriter();
        if(request.arguments().hasNext()){
            writer.println("501 Syntax error (no parameters allowed)");
        } else {
            writer.println("220 Ready to start TLS");
            java.security.Security.addProvider( new com.sun.net.ssl.internal.ssl.Provider());
            //     System.setProperty("javax.net.debug", "ssl,handshake");
            SMTPProtocolInstance smtpProtocol = (SMTPProtocolInstance)protocol; 
            SSLSocketFactory factory = 
            	(SSLSocketFactory)smtpProtocol.getProperty(SMTPConstants.SSL_SOCKET_FACTORY);
            Socket oldSocket = smtpProtocol.getServerThread().getSocket();
            final SSLSocket newSocket = 
                (SSLSocket)factory.createSocket(oldSocket,
                                                oldSocket.getLocalAddress().getHostName(),
                                                oldSocket.getLocalPort(),
                                                true);
            newSocket.setNeedClientAuth(protocol.getPropertyBool(SMTPConstants.
                                                                 REQUIRE_CLIENT_CERT));
            newSocket.setUseClientMode(false);
            final SMTPProtocolInstance tempProtocol = (SMTPProtocolInstance)protocol;
            newSocket.addHandshakeCompletedListener(new HandshakeCompletedListener(){
                public void handshakeCompleted(HandshakeCompletedEvent event){
                    jblog.debug("HS completed");
                    tempProtocol.setSecure(true);
                    
                    //Reset the state but carry over the addresses connected as this is needed
                    //by MailHeadersImpl
                    Object localAddr = tempProtocol.getState(ServerThread.STATE_LOCAL_ADDRESS);
                    Object remoteAddr = tempProtocol.getState(ServerThread.STATE_CLIENT_ADDRESS);
                    tempProtocol.resetState();
					tempProtocol.setState(ServerThread.STATE_LOCAL_ADDRESS, localAddr);
					tempProtocol.setState(ServerThread.STATE_CLIENT_ADDRESS, remoteAddr);
                    
                    
                    jblog.debug("State reset");
                    Certificate certs[] = null;
                    try{
                        certs = newSocket.getSession().getPeerCertificates();
                        int j;
                        for(j=certs.length -1; j >= 0; j--){
                            jblog.debug(certs[j]);
                        }
                        //TODO: Put some info about the Certificate in Mail-Header
                    }catch(SSLPeerUnverifiedException e){
                        e.printStackTrace();
                    }
                }
            });
            
            ServerThread serverThread = ((SMTPProtocolInstance)protocol).getServerThread(); 
            serverThread.setSocket(newSocket);
        }
        writer.flush();
        return response;
    }
}
