/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import org.jboss.logging.Logger;
import org.jboss.mail.Protocol;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPProtocolInstance;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;


/**
 * Proxy for {@link org.jboss.mail.smtp.handlers.SMTPHandler}. 
 * Sends "530 Must issue a STARTTLS command first" if REQUIRE_STARTTLS
 * is configured and the connection is not secure.<br>
 * Otherwise it delegates to its {@link org.jboss.mail.smtp.handlers.SMTPHandler}
 * @author Michael Krause
 */
public class RequireSTARTTLSProxy implements SMTPHandler {

    private SMTPHandler handler;
    private static final Logger jblog = Logger.getLogger(RequireSTARTTLSProxy.class);

    public RequireSTARTTLSProxy(SMTPHandler handler){
        this.handler = handler;
    }  

    public SMTPResponse handleRequest(OutputStream out, SMTPRequest request, Protocol protocol) throws IOException {
        if(((SMTPProtocolInstance)protocol).
           getPropertyBool(SMTPConstants.REQUIRE_STARTTLS) && 
           (!((SMTPProtocolInstance)protocol).isSecure())){
            SMTPResponse response = new SMTPResponse(request, out, protocol);
            PrintWriter writer = response.getWriter();
            jblog.warn("Sending 530 Must issue a STARTTLS command first");
            writer.println("530 Must issue a STARTTLS command first");
            writer.flush();
            return response;
        }else{
            return handler.handleRequest(out, request, protocol);
        }
    }
}





