/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import org.jboss.mail.Protocol;

import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;

import java.io.IOException;
import java.io.OutputStream;

/**
 * SMTPHandler interface defines the method to which requests are passed.  It is called by the 
 * SMTPProtocol in order to actually handle the request.  New handlers can be added (presently 
 * statically, though extensibility is feasible) easily to the SMTPHandlers class. 
 * 
 * @author Andrew C. Oliver
 */
public interface SMTPHandler {
    /**
     * Handle the processing of the request
     * @param out outputstream in which messages and responses are written
     * @param request which is to be processed
     * @param protocol instance which delegated the request
     * @return SMTPResponse providing information about the processing of the request.
     * @throws IOException
     */
    SMTPResponse handleRequest(
        OutputStream out,
        SMTPRequest request,
        Protocol protocol)
        throws IOException;
}
