/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.util.Map;
import java.util.HashMap;

/**
 * This class forms a handler collection which allows the caller to receive a statically held instance
 * of the handler Map.  Handlers are singleton instances in nature delegating all session information
 * up to the Protocol handler.  Handlers are mapped in such a way that the key value from their "COMMAND"
 * member determines their use. 
 * 
 * @author Andrew C. Oliver
 * @author Kabir Khan 
 */
public class SMTPHandlers {
    private static Map handlers;

    /**
     * return the map of handlers
     * @return Map containing the handlers mapped to their keys
     */
    public static Map instance() {
        if (handlers == null) {
            constructHandlers();
        }
        return handlers;
    }

    /**
     * static internal initializer which is called to construct the map of handlers in the event the
     * mapping is null.
     */
    private static synchronized void constructHandlers() {
        if (handlers == null) {
            handlers = new HashMap();

            handlers.put(CmdHELO.COMMAND, new RequireSTARTTLSProxy(new CmdHELO()));

            handlers.put(CmdEHLO.COMMAND, new CmdEHLO());

            handlers.put(CmdQUIT.COMMAND, new CmdQUIT());

            handlers.put(CmdMAIL.COMMAND, new RequireSTARTTLSProxy(new CmdMAIL()));

            handlers.put(CmdRCPT.COMMAND, new RequireSTARTTLSProxy(new CmdRCPT()));

            handlers.put(CmdAUTH.COMMAND, new RequireSTARTTLSProxy(new CmdAUTH()));

            handlers.put(CmdDATA.COMMAND, new RequireSTARTTLSProxy(new CmdDATA()));

            handlers.put(CmdSTARTTLS.COMMAND, new CmdSTARTTLS());
            
            handlers.put(CmdRSET.COMMAND, new CmdRSET());

			handlers.put(CmdNOOP.COMMAND, new CmdNOOP());

 			handlers.put(CmdHELP.COMMAND, new CmdHELP());

 			handlers.put(CmdVRFY.COMMAND, new CmdVRFY());
 			
 			handlers.put(CmdEXPN.COMMAND, new CmdEXPN());
       }
    }
}

