package org.jboss.mail.smtp.sender;

import java.net.UnknownHostException;

import org.jboss.mail.ServiceProxyFactory;
import org.jboss.mail.message.Mail;
import org.jboss.mail.message.MailAddress;
import org.jboss.system.ServiceMBean;
import org.w3c.dom.Element;

/**
 * SMTPSender MBean provides the SMTP client funcionality for sending mails to other servers.  Messages that must be
 * sent remotely must be sent via this mbean to another SMTP server.  Potentially it may relay them as well.  In the 
 * present version we only intend to allow authenticated local users to relay messages.  Later we'll allow defined 
 * relays.  
 * @todo revise this comment when appropriate (12-31-03)
 * @author Andrew C. Oliver
 */
public interface SMTPSenderMBean extends ServiceMBean, ServiceProxyFactory {
    /**
     * @return DNS servers to use
     */
    Element getDNSServers();
    /**
     * set the DNS servers to use for looking up MX records
     * @param servers
     */
    void setDNSServers(Element servers);
    
    /**
     * add a DNS server
     * @param dns server name
     * @see #setDNSServers(Element)
     * @throws RuntimeException in the event the server cannot be resolved
     */
    void addDNSServer(String dns) throws UnknownHostException;
    
    /**
     * remove a DNS server from the list
     * @param dns server name
     * @throws RuntimeException if the server isn't used
     */
    void removeDNSServer(String dns);
    
    /**
     * enable the DNS cache
     * @param enabled boolean value 
     */
    void setCacheEnabled(boolean enabled);
    /**
     * @return is the DNS cache enabled?
     */
    boolean getCacheEnabled();
    
    
    /**
     * lookup the 
     * @return ...
     */
    public String[] mxLookup(String address);
    
    void clearDNSCache();
    
    /**
     * send mail to all of its receipients except those listed
     * @param message to send
     * @param exclude list of receipients not to send it to (Mail.getTo() - excluded = sendees) 
     * @return result of sending
     */
    SMTPResult[] send(Mail message, MailAddress[] exclude);
    
	void testSend(String to, String from, String subject, String message);

	
	/** Send mail to one recipient 
	 * @param to recipient to receive the mail
	 * @param message message to be sent
	 */
	SMTPResult send(MailAddress to, Mail message);
	
}
