/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.userrepository;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.jboss.console.server.MetaInfo;
import org.jboss.mail.ServiceProxy;
import org.jboss.system.ServiceMBean;
import org.jboss.system.ServiceMBeanSupport;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This UserRepository is merely configured via the JBoss XML Mbean config with &lt;users&gt;,
 * &lt;user&gt;, &lt;id&gt;, and &lt;password&gt; elements.  These are statically configured or can be 
 * ammended via the JMX console.
 * @author Andrew C. Oliver 
 */
public class StaticUserRepository
    extends ServiceMBeanSupport
    implements ServiceMBean, StaticUserRepositoryMBean, Serializable  {

    private Map users;
    private String name;

    public StaticUserRepository() {
        users = new HashMap();
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#produceServiceProxy()
     */
    public ServiceProxy produceServiceProxy() {
        return new StaticUserRepositoryProxy(users);
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#setUsers(org.w3c.dom.Element)
     */
    public void setUsers(Element users) {
        NodeList nodes = users.getElementsByTagName("user");
        Map pusers = new HashMap(nodes.getLength());
        for (int k = 0; k < nodes.getLength(); k++) {
            String pname = null;
            String pval = null;
            Node node = nodes.item(k);
            NodeList subnodes = node.getChildNodes();
            for (int x = 0; x < subnodes.getLength(); x++) {
                Node subnode = subnodes.item(x);
                if (subnode.getNodeName().equals("id")) {
                    pname = subnode.getFirstChild().getNodeValue();
                } else if (subnode.getNodeName().equals("password")) {
                    pval = subnode.getFirstChild().getNodeValue();
                } else {
                    throw new RuntimeException("user id and password pairs are all that are needed/permitted");
                }
            }
            if (pname == null || pval == null) {
                throw new RuntimeException("users need passwords and passwords need users...");
            }
            pusers.put(pname, pval);
        }
        this.users = pusers;
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#getUsers()
     */
    public Element getUsers() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {

            e.printStackTrace();
        }
        DOMImplementation impl = builder.getDOMImplementation();

        DocumentType DOCTYPE = impl.createDocumentType("non", "non", "non");
        Document doc = impl.createDocument("non", "non", DOCTYPE);
        Element retval = doc.createElement("users");
        Iterator i = null;
        for (i = users.keySet().iterator(); i.hasNext();) {
            String key = (String) i.next();
            String val = (String) users.get(key);
            Element user = doc.createElement("user");
            Element id = doc.createElement("id");
            Element pw = doc.createElement("password");
            id.appendChild(doc.createTextNode(key));
            pw.appendChild(doc.createTextNode(val));
            user.appendChild(id);
            user.appendChild(pw);
            retval.appendChild(user);
        }

        return retval;
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#addUser(java.lang.String, java.lang.String)
     */
    public void addUser(String user, String password) {
        users.put(user, password);
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#listUsers()
     */
    public String[][] listUsers() {
        String[][] retval = new String[users.size()][2];
        int k = 0;
        for (Iterator i = users.keySet().iterator(); i.hasNext(); k++) {
            retval[k][0] = (String) i.next();
            retval[k][1] = (String) users.get(retval[k][0]);
        }
        return retval;
    }

    /**
     * @return String with a comma /line delimited list of users
     */
    public String userList() {
        String[][] list = listUsers();
        String retval = "";
        for (int k = 0; k < list.length; k++) {
            retval += list[k][0];
            //don't forget that now StringBuffers automatically used
            retval += ", "; //you no longer have to do it the hard way.
            retval += list[k][1];
            retval += "\n";
        }
        return retval;
    }

	/* (non-Javadoc)
	 * @see org.jboss.mail.userrepository.StaticUserRepositoryMBean#getMetaInfo()
	 */
	public MetaInfo beanMetaInfo() {
		return createMetaInfo();
	}

	/**
	 * @return
	 */
	private MetaInfo createMetaInfo() {
		// TODO Auto-generated method stub
		return new MetaInfoImpl(getBytesForIcon());
	}
	
	private byte[] getBytesForIcon() {
		byte[] retval = null;
		InputStream stream = new BufferedInputStream(this.getClass().getClassLoader().getResourceAsStream("users.jpg"));
		byte[] b = new byte[812];
		List bytes = new ArrayList();
		int size = 0;
		int s = 1;
		try {
			while (s > 0) {
				System.out.println("reading "+s);
				s = stream.read(b); 
				if (s > 0) {
				  bytes.add(b);
				}
				size += s; 
				b = new byte[812];
			}
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		retval = new byte[size];
		int loc = 0;
		for (int i = 0; i < bytes.size(); i++) {
			byte[] add = (byte[])bytes.get(i);
			if (loc + add.length < size) {
				System.arraycopy(add,0,retval,loc, add.length);
				loc +=add.length;
			} else {
				System.arraycopy(add,0,retval,loc,size-loc);
			}
				
		}
		return retval;
	}
}


