/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.userrepository;

import org.jboss.console.server.MetaInfo;
import org.jboss.mail.ServiceProxy;
import org.jboss.mail.ServiceProxyFactory;
import org.jboss.system.ServiceMBean;
import org.w3c.dom.Element;

/**
 * This UserRepository is merely configured via the JBoss XML Mbean config with &lt;users&gt;,
 * &lt;user&gt;, &lt;id&gt;, and &lt;password&gt; elements.  These are statically configured or can be 
 * ammended via the JMX console.
 * @author Andrew C. Oliver
 */
public interface StaticUserRepositoryMBean extends ServiceMBean, ServiceProxyFactory, java.io.Serializable {
    /*
     *  (non-Javadoc)
     * @see org.jboss.mail.ServiceProxyFactory#produceServiceProxy()
     */
    public ServiceProxy produceServiceProxy();
       
    /** 
     * set the users via a DOM element
     * @param users element
     */
    public void setUsers(Element users);
    /**
     * @return the list of users via a DOM element
     */
    public Element getUsers();
    /**
     * @return {{user,password}{user,password}}
     */
    public String[][] listUsers();
    /**
     * @return big fat string containing user,password\n till we run out
     */
    public String userList();

    /**
     * add a user
     * @param user name of user
     * @param password of user
     */
    public void addUser(String user, String password);
    
    public MetaInfo beanMetaInfo();
    
}
