/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.util;

import org.jboss.logging.Logger;

import java.util.Iterator;
 
import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;

/**
 * Utility to handle JMX stuff so that I don't have to think very hard about it
 * @author Andrew C. Oliver
 */
public class JBMailJMXUtil {
    private static final Logger log = Logger.getLogger(JBMailJMXUtil.class);

    private static boolean debug;
    public static synchronized void switchToDebug() {
        JBMailJMXUtil.debug = true;
    }

    /**
     * @param JMX name of the MBean
     * @return Object implementing ServiceProxy
     */
    public static Object getService(String key) throws Exception {
        ObjectName name = new ObjectName(key);
        return getService(name);
    }

    /**
     * @return the MBeanServer whcih has "jboss" as the default domain
     */
    public static MBeanServer locateJBoss() {
        for (Iterator i = MBeanServerFactory.findMBeanServer(null).iterator();
            i.hasNext();
            ) {
            MBeanServer server = (MBeanServer) i.next();
            if (server.getDefaultDomain().equals("jboss"))
                return server;
        }
        throw new IllegalStateException("JBoss MBeanServer not found!");
    }

    /**
     * @return
     */
    private static MBeanServer fakeMBeanServer() {
        // MBeanServer retval = new
        return null;
    }

    /**
     * @param JMX objectname of the MBean
     * @return Object implementing ServiceProxy
     */
    public static Object getService(ObjectName name) throws Exception {
        MBeanServer srv = locateJBoss();       
        Object retval = srv.invoke(name, "produceServiceProxy", new Object[] {
        }, new String[] {
        });
        return retval;
    }    

}
