/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.util;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.Properties;

import javax.mail.MessagingException;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimePart;
import javax.mail.internet.MimeUtility;

import org.jboss.mail.message.Mail;

/**
 * Utility class for handling Mime.
 * 
 * @author <a href="kabirkhan@bigfoot.com">Kabir Khan </a>
 */

public class MimeUtil {

	/**
	 * Returns the plain text body of a mail, in the proper encoding
	 * 
	 * @param mail
	 * @return
	 */
	public static String getPlainTextBody(Mail mail) {
		try {
			Properties props = new Properties();
			Session session = Session.getInstance(props);
			BufferedInputStream in = new BufferedInputStream(
					new ByteArrayInputStream(mail.getRawBytes()));
			MimeMessage message = new MimeMessage(session, in);

			MimePart messagePart = getTextPart(message);

			String contentType = messagePart.getContentType();
			InputStream is = messagePart.getInputStream();

			int charsetStart = contentType.indexOf("charset=");
			int charsetEnd = -1;

			if (charsetStart >= 0) {
				charsetStart += "charset=".length();
				charsetEnd = contentType.indexOf(";", charsetStart);
			}

			StringBuffer body = new StringBuffer();
			String charset = charsetStart >= 0 && charsetEnd >= 0 ? contentType
					.substring(charsetStart, charsetEnd) : "US-ASCII";
			charset = MimeUtility.javaCharset(charset);
			BufferedReader reader = new BufferedReader(new InputStreamReader(
					is, charset));
			String line = reader.readLine();
			while (line != null) {
				if (!line.equals(".")) {
					body.append(line + "\r\n");
				}
				line = reader.readLine();
			}
			return body.toString();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return null;
	}

	public static MimePart getTextPart(MimePart messagePart)throws IOException, MessagingException, UnsupportedEncodingException{
		
		Object content = messagePart.getContent();
		
		if (content instanceof Multipart) {
			// Look for content-type: text/plain

			for (int i = 0; i < ((Multipart) content).getCount(); i++) {

				messagePart = (MimePart)((MimeMultipart) content).getBodyPart(i);
				String contentType = messagePart.getContentType();

				if (contentType.startsWith("text/plain")) {
					return messagePart;
				}else if (contentType.startsWith("multipart/")){
					
					try{
						messagePart = getTextPart(messagePart);
						return messagePart;
					}
					catch(UnsupportedEncodingException e){
						
					}
				}
			}
		}else{
			return messagePart;
		}
		
		throw new UnsupportedEncodingException("No text/plain part could be found in the mail");
	}

}