/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.BufferedInputStream;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.InputStream;
import java.util.Properties;

import javax.management.MBeanServer;
import javax.management.MBeanServerFactory;
import javax.management.ObjectName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

//import org.jboss.console.twiddle.TwiddleProtocol;
import org.jboss.mail.domaingroup.DomainGroup;
import org.jboss.mail.maillist.memory.MemoryMailListManager;
import org.jboss.mail.maillistener.SysOutMailListener;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPProtocol;
import org.jboss.mail.userrepository.StaticUserRepository;
import org.jboss.mail.util.JBMailJMXUtil;
import org.jboss.system.server.ServerConfig;
import org.jboss.system.server.ServerImpl;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;

/**
 * Just some plumbing used to run unit tests outside of the appserver.
 *
 * @author Andrew C. Oliver
 */ 
public class MBeanServerUtil {
    private static MBeanServerFactory factory;
    private static MBeanServer server;
    private static boolean alreadyrun;
	private static boolean sysoutreg;
	private static boolean surreg;
	private static boolean smtpreg;
	private static boolean twidreg;
	private static boolean domaingroupreg;
	private static boolean memorylistservreg;
	
    public static void configureMBeanServerFactory() throws IllegalStateException, Exception {
    	System.out.println("configureMBeanServerFactory");
    	if (alreadyrun) return;
    	System.out.println("not returning from already run");
       // server = MBeanServerFactory.createMBeanServer();
       ServerImpl impl = new ServerImpl();
       System.out.println("got the server impl");
       Properties prop = new Properties();
       prop.setProperty(ServerConfig.HOME_DIR,(new File(".")).getAbsolutePath());
       prop.setProperty(ServerConfig.ROOT_DEPLOYMENT_FILENAME,"src/META-INF/depends-service.xml");
       prop.setProperty(ServerConfig.SERVER_CONFIG_URL,"file:"+(new File(".")).getAbsolutePath());
       System.out.println("done setting up properties");
       impl.init(prop);
       System.out.println("impl.init is finished");
       impl.start();
       System.out.println("impl.start done");
       alreadyrun = true;
       System.out.println("done configureMBeanServerFactory"); 
    }
    
    public static void setupSysout() throws Exception {
    	registerStaticUserRepository();
    	registerSysOutMailListener();
    	registerSMTPProtocol();
    }
    
    public static void registerStaticUserRepository() throws Exception {
    	System.out.println("register static user repository");
    	if (surreg) return;
		MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
		ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=UserRepository");
		StaticUserRepository sur = new StaticUserRepository();
		sur.addUser("acoliver","test");
		sur.addUser("test","testpw");
		sur.addUser("jboss","jbosspw");
		mbserver.registerMBean(sur, oname);
		surreg = true;
    }
    
	public static void registerSysOutMailListener() throws Exception {
		if (sysoutreg) return;
		MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
		ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=MailListener");
		SysOutMailListener soml = new SysOutMailListener();
		mbserver.registerMBean(soml, oname);
		sysoutreg = true;		
	}
    
	public static void registerDomainGroup() throws Exception {
		if (domaingroupreg) return;
		MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
		ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=DomainGroup,group=Local");
		DomainGroup dg = new DomainGroup();
		dg.add("badmojo");
		dg.add("localhost");
		dg.add("badmojo.superlinksoftware.com");
		dg.add("superlinksoftware.com");
		mbserver.registerMBean(dg, oname);
		domaingroupreg = true;		
	}
    
    public static void registerMemoryListServ() throws Exception{
    	if (memorylistservreg)return;
    	MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
    	ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=MailListManager");
    	MemoryMailListManager mls = new MemoryMailListManager();
    	
    	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder = null;
		try {
			builder = factory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {

			e.printStackTrace();
		}

		String config = "<mailingLists>"
			+ "	<mailingList>"
			+ "		<listAddress>alist@localhost.localdomain</listAddress>"
			+ "		<property><name>replyToList</name><value>false</value></property>" 
			+ "		<property><name>subjectPrefix</name><value>Test List</value></property>" 
			+ "		<property><name>prefixAutoBracketed</name><value>false</value></property>" 
			+ "		<property><name>attachmentAllowed</name><value>true</value></property>"
			+ "		<property><name>membersOnly</name><value>false</value></property>"
			+ "		<members>"
			+ "			<member>1@abc.com</member>"
			+ "			<member>2@def.com</member>"
			+ "		</members>"
			+ "	</mailingList>"
			+ "	<mailingList>"
			+ "		<listAddress>blist@localhost.localdomain</listAddress>"
			+ "		<members>"
			+ "			<member>a@abc.com</member>"
			+ "			<member>b@def.com</member>"
			+ "			<member>c@ghi.com</member>"
			+ "		</members>"
			+ "	</mailingList>"
			+ "</mailingLists>";

		
		byte[] bytes =config.getBytes();
		InputStream in = new BufferedInputStream(new ByteArrayInputStream(bytes));
		Document doc = builder.parse(in);
		Element mailingLists = (Element)doc.getFirstChild(); 
		mls.setMailingLists(mailingLists);
		
    	mbserver.registerMBean(mls, oname);
    	memorylistservreg = true;
    	
    	
    }
    
    public static void registerSMTPProtocol() throws Exception {
    	if (smtpreg) return;
		MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
		ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=SMTPProtocol");
		SMTPProtocol sp = new SMTPProtocol();
		sp.setProperties(createProps());
		mbserver.registerMBean(sp, oname);
		smtpreg = true;
    }
    
/*    public static void registerTwiddleProtocol() throws Exception {
    	if (twidreg) return;
		MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
		ObjectName oname = new ObjectName("jboss.mail:type=MailServices,name=TwiddleProtocol");
		TwiddleProtocol tp = new TwiddleProtocol();
	//	tp.setProperties(createProps());
		mbserver.registerMBean(tp, oname);
		twidreg = true;
    }*/
    
    public static void startServerService(Server bean) throws Exception {
        bean.startService();
    }
    
    public static void main(String[] args) throws Exception {
        MBeanServerUtil.configureMBeanServerFactory();
    }
    
 
		private static Element createProps() {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder = null;
			try {
				builder = factory.newDocumentBuilder();
			} catch (ParserConfigurationException e) {

				e.printStackTrace();
			}
			DOMImplementation impl = builder.getDOMImplementation();

			DocumentType DOCTYPE = impl.createDocumentType("non", "non", "non");
			Document doc = impl.createDocument("non", "non", DOCTYPE);
			Element retval = doc.createElement("properties");
			Element property = getProperty(doc, SMTPConstants.SERVER_NAME, "localhost");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.AUTH_REQUIRED, "true");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.AUTH_METHODS, "AUTH LOGIN PLAIN");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.USER_REPOSITORY, "jboss.mail:type=MailServices,name=UserRepository");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.VERIFY_IDENTITY, "true");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.MAX_MSG_SIZE, "1000000");
			retval.appendChild(property);
			property = getProperty(doc, SMTPConstants.MAIL_LISTENERS, new String[]{"jboss.mail:type=MailServices,name=MailListener"});
			retval.appendChild(property);
        

			return retval;
		}
		
	private static Element getProperty(Document doc, String keyval, String[] strings) {
		Element property = doc.createElement("property");
		 Element key = doc.createElement("name");
		 key.appendChild(doc.createTextNode(keyval));
		 Element values = doc.createElement("values");
		 for (int k = 0; k < strings.length; k++) {
			Element value = doc.createElement("value");
			value.appendChild(doc.createTextNode(strings[k]));
			values.appendChild(value);
		 }         
		 property.appendChild(key);
		 property.appendChild(values);
		 return property;

	}

	private static Element getProperty(Document doc, String keyval, String val) {
		Element property = doc.createElement("property");
		 Element key = doc.createElement("name");
		 key.appendChild(doc.createTextNode(keyval));
		 Element value = doc.createElement("value");
		 value.appendChild(doc.createTextNode(val));  
		 property.appendChild(key);
		 property.appendChild(value);           
		 return property;
	}

		
}
    
