/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import junit.framework.TestCase;

/**
 * Tests that the SizeLimitedInputStream limits the size correctly.  SLIS is ripped from Apache JAMES.
 * (thanks!)  -- The unit test is my own brew.
 * @author Andrew C. Oliver
 */
public class TestSizeLimitedInputStream extends TestCase {
    InputStream bin;
    public TestSizeLimitedInputStream(String name)
    {
        super (name);
    }
   
    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestSizeLimitedInputStream.class);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        bin = createBin();
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    /*
     * Class to test for int read()
     */
    public void testRead() throws IOException {
        InputStream in = new SizeLimitedInputStream(bin, 50);
        for (int k = 0; k < 50; k++) {
            int r = in.read();
        }

        in = new SizeLimitedInputStream(bin, 50);
        int n = TestHelper.bytesReadBeforeException(in);
        assertTrue("num bytes read should be 50 was " + n, n == 50);

    }

    /*
     * Class to test for int read(byte[], int, int)
     */
    public void testReadbyteArray() throws IOException {
        InputStream in = new SizeLimitedInputStream(bin, 500);
        byte[] b = new byte[1000];
        int k = in.read(b, 0, 500);

        in = new SizeLimitedInputStream(bin, 500);
        b = new byte[500];
        k = in.read(b, 0, b.length);
    }

    /**
     * @return a byteArrayInputStream with 1000 bytes
     */
    private InputStream createBin() {
        byte[] oneK = new byte[1000];
        for (int k = 0; k < oneK.length; k++) {
            oneK[k] = (byte) k;
        }
        ByteArrayInputStream result = new ByteArrayInputStream(oneK);
        return result;
    }

}

/**
 * Helper class to let us count the bytes read before the exception.  We need this so that Junit doesn't
 * barf on exception.
 * @author Andrew C. Oliver
 */
class TestHelper {
    /**
     * @param in - byte array to read until exception
     * @return number of bytes read before exception or -1 if none thrown
     */
    public static int bytesReadBeforeException(InputStream in) {
        int k = 0;
        try {
            while (in.read() != -1) {
                k++;
            }
            k = -1;
        } catch (Exception e) {
            k--;
        }
        return k;
    }
}
