/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.dev;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

/**
 * Test the condition part of the script runner.  Makes sure that it can parse the basic branchings.
 * @author Andrew C. Oliver
 */
public class TestCondition extends TestCase {

	public static void main(String[] args) {
		junit.textui.TestRunner.run(TestCondition.class);
	}

	/*
	 * test a basic condition list
	 */
	public void testConditionList() {
		List list = new ArrayList(2); 
		list.add("S: this is a test");
		list.add("C: success");
		Condition c = new Condition(list);
        String expected = "success";
        String result = c.respond("this is a test");
		assertEquals("expected "+expected+"got "+result,expected,result);
	}
	
	/**
	 * test a branched condition (if(x || y) then z)
	 */
	public void testMultiHeadConditionList(){
		List list = new ArrayList(2); 
		list.add("S1: this is a test");
		list.add("S2: this is a dumb test");
		list.add("C: success");
		Condition c = new Condition(list);
		String expected = "success";
		String result = c.respond("this is a test");
		assertEquals("expected "+expected+"got "+result,expected,result);
		result = c.respond("this is a dumb test");
		assertEquals("expected "+expected+"got "+result,expected,result);
	}
	
	/**
	 * test a branched condition (if(w){ x } else if (y) {z})
	 *
	 */
	public void testMultiBranchConditionList(){
		List list = new ArrayList(2); 
		list.add("S1: this is a test");
		list.add("C1: Yup, nice test");
		list.add("S2: this is a dumb test");
		list.add("C2: Yup, dumb test");
		Condition c = new Condition(list);
		String expected = "Yup, nice test";
		String result = c.respond("this is a test");
		assertEquals("expected "+expected+"got "+result,expected,result);
		expected = "Yup, dumb test";
		result = c.respond("this is a dumb test");
		assertEquals("expected "+expected+"got "+result,expected,result);
	}	
	
}
