/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.domaingroup;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;

import junit.framework.TestCase;


/**
 * Test for DomainGroupMBean
 * @author Andrew C. Oliver
 */
public class TestDomainGroup extends TestCase {
    DomainGroupMBean domaingroup;
    public static void main(String[] args) {
    //    junit.textui.TestRunner.run(TestDomainGroup.suite());
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
        domaingroup = new DomainGroup();
        domaingroup.start();
    }
    
    public void testNormal() throws Exception {
        domaingroup = new DomainGroup();
        domaingroup.setDomains(createDomains());
        String[] domains = domaingroup.listDomains();
        assertTrue("localhost was in domaingroup", isIn(domains,"localhost"));
        assertTrue("jboss.org was in domaingroup", isIn(domains,"jboss.org"));
        assertTrue("mail.jboss.org was in domaingroup", isIn(domains,"mail.jboss.org"));                
    }

    /**
     * @return list of domains for testing
     */
    private Element createDomains() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {

            e.printStackTrace();
        }
        DOMImplementation impl = builder.getDOMImplementation();

        DocumentType DOCTYPE = impl.createDocumentType("non", "non", "non");
        Document doc = impl.createDocument("non", "non", DOCTYPE);
        Element retval = doc.createElement("domains");

        Element domain = doc.createElement("domain");
        domain.appendChild(doc.createTextNode("localhost"));
        retval.appendChild(domain);

        domain = doc.createElement("domain");
        domain.appendChild(doc.createTextNode("jboss.org"));
        retval.appendChild(domain);
        
        domain = doc.createElement("domain");
        domain.appendChild(doc.createTextNode("mail.jboss.org"));
        retval.appendChild(domain);        
        
        return retval;
    }

    public void testAddDomain() throws Exception {
        domaingroup.add("localhost");
        domaingroup.add("jboss.org");
        domaingroup.add("mail.jboss.org");
        String[] domains = domaingroup.listDomains();
        assertTrue("localhost was in domaingroup", isIn(domains,"localhost"));
        assertTrue("jboss.org was in domaingroup", isIn(domains,"jboss.org"));
        assertTrue("mail.jboss.org was in domaingroup", isIn(domains,"mail.jboss.org"));        
    }
    
    public void testIsInGroup() throws Exception {
        domaingroup.add("localhost");
        domaingroup.add("jboss.org");
        domaingroup.add("mail.jboss.org");
        assertTrue("localhost was in domaingroup", domaingroup.isInGroup("localhost"));
        assertTrue("jboss.org was in domaingroup", domaingroup.isInGroup("jboss.org"));
        assertTrue("mail.jboss.org was in domaingroup", domaingroup.isInGroup("mail.jboss.org"));        
    }
    
    private boolean isIn(String[] strings, String string) {
        for (int k = 0; k < strings.length; k++) {
            if(strings[k].equals(string)) {
                return true;
            }
        }
        return false;
    }

    /*
     * @see TestCase#tearDown()
     */
    protected void tearDown() throws Exception {
        super.tearDown();
    }
}
