/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillistener;

import java.io.Serializable;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.management.ObjectName;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.TestCase;

import org.jboss.mail.MBeanServerUtil;
import org.jboss.mail.message.MailAddress;
import org.w3c.dom.DOMImplementation;
import org.w3c.dom.Document;
import org.w3c.dom.DocumentType;
import org.w3c.dom.Element;

import com.sun.naming.internal.ResourceManager;

/**
 * Tests the JMS Mail Listener
 * TODO: Add queue test to make sure the right messages get put on the right queues
 * @author Andrew C. Oliver
 */
public class TestJMSMailListener extends TestCase {

    private JMSMailListenerMBean maillistener;

    public TestJMSMailListener(String name)
    {
        super (name);
		try {
			System.out.println("TestJMSMailListener Context.PROVIDER_URL: " + ResourceManager.getInitialEnvironment(null).get(Context.PROVIDER_URL));
		} catch (NamingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
   
    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestJMSMailListener.class);
    }

    /*
     * @see TestCase#setUp()
     */
    protected void setUp() throws Exception {
		//Set up the DomainGroup needed by the maillistener
		MBeanServerUtil.configureMBeanServerFactory();
        
        InitialContext ctx = new InitialContext();
		ctx.bind("java:/ConnectionFactory", new TestQueueConnectionFactory());
		//For some reason:
		//	ctx.bind("queue/post", "classcastexceptioninabox");
		//causes a NameNotFoundException "queue not bound"
		Context qctx = ctx.createSubcontext("queue");
		qctx.bind("post", new TestQueue("post"));
		qctx.bind("off", new TestQueue("off"));    

    	maillistener = new JMSMailListener("java:/ConnectionFactory","java:/ConnectionFactory");
        maillistener.setDestinationType("queue");

		MBeanServerUtil.registerDomainGroup();
		maillistener.setDomainGroup(new ObjectName("jboss.mail:type=MailServices,name=DomainGroup,group=Local"));
		maillistener.setOnServerPostDestination("post");
		maillistener.setOffServerPostDestination("off");
        maillistener.start();
    }

	/*
	 * @see TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		maillistener.stop();
		InitialContext ctx = new InitialContext();
		ctx.unbind("queue");
		ctx.unbind("java:/ConnectionFactory");
	}

    public void testSetOnServerPostDestination() throws Exception {
        maillistener.setDestinationType("queue");
        maillistener.setOnServerPostDestination("post");
        maillistener.setOffServerPostDestination("off");
        assertEquals("post", maillistener.getOnServerPostDestination());
        assertFalse(
            "on server and offserver should be not equal",
            maillistener.getOnServerPostDestination().equals(
                maillistener.getOffServerPostDestination()));
	}

    public void testSetOnServerDomains() throws Exception {
        maillistener.setDestinationType("queue");
        maillistener.setOnServerPostDestination("post");
        maillistener.setOffServerPostDestination("off");

        MailAddress ma = MailAddress.parseSMTPStyle("<noone@localhost>");
        String queue = maillistener.getDestinationForMailAddress(ma);
        assertEquals(queue, "post");
        ma = MailAddress.parseSMTPStyle("<someone@spamreceipient.org>");
        queue = maillistener.getDestinationForMailAddress(ma);
        assertEquals(queue, "off");
    }

    /**
     * @return
     */
    private Element createDomainsList() {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException e) {

            e.printStackTrace();
        }
        DOMImplementation impl = builder.getDOMImplementation();

        DocumentType DOCTYPE = impl.createDocumentType("non", "non", "non");
        Document doc = impl.createDocument("non", "non", DOCTYPE);
        Element retval = doc.createElement("domains");

        Element domain = doc.createElement("domain");
        domain.appendChild(doc.createTextNode("localhost"));
        retval.appendChild(domain);

        domain = doc.createElement("domain");
        domain.appendChild(doc.createTextNode("nowhere.org"));
        retval.appendChild(domain);

        return retval;
    }


}


class TestQueueConnectionFactory implements QueueConnectionFactory, Serializable{

	public QueueConnection createQueueConnection() throws JMSException {
		// TODO Auto-generated method stub
		return null;
	}

	public QueueConnection createQueueConnection(String arg0, String arg1)
			throws JMSException {
		// TODO Auto-generated method stub
		return null;
	}
};

class TestQueue implements Queue, Serializable{
	String queueName;
	public TestQueue(String queueName){
		this.queueName = queueName;
	}

	public String getQueueName() throws JMSException {
		// TODO Auto-generated method stub
		return null;
	}

}
