/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.message;

import junit.framework.TestCase;

/**
 * Tests that the MailAddress parses valid addresses and property returns invalid state for invalid 
 * addresses and that the properties (user/domain) are what we imagine.
 * @author Andrew C. Oliver
 */
public class TestMailAddress extends TestCase {

    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestMailAddress.class);
    }

    public TestMailAddress(String name){
        super(name);
    }

    public void testIsValid() {
        MailAddress addr = MailAddress.parseSMTPStyle("invalid address");
        System.err.println(addr.getUser());
        System.err.println(addr.getDomain());
        assertTrue("addr isvalid should be false "+addr.isValid(), addr.isValid() == false);
        addr = MailAddress.parseSMTPStyle("<noone@nowhere.com>");
        assertTrue("addr isvalid should be true "+addr.isValid(), addr.isValid() == true);
		assertEquals("<noone@nowhere.com>", addr.toString());
    }
    
	public void testEmtpyAddress(){
		MailAddress addr = MailAddress.parseSMTPStyle("<>");
		assertTrue("addr isvalid should be false "+addr.isValid(), addr.isValid() == false);
		assertTrue("addr isempty should be true "+addr.isEmpty(), addr.isEmpty() == true);
	}

	public void testEmtpyFromAddress(){
		MailAddress addr = MailAddress.parseSMTPStyle("<>", true);
		assertTrue("addr isvalid should be true "+addr.isValid(), addr.isValid() == true);
		assertTrue("addr isempty should be true "+addr.isEmpty(), addr.isEmpty() == true);
		assertEquals("<>", addr.toString());
	}

    public void testGetUser() {
        MailAddress addr = MailAddress.parseSMTPStyle("<testaddr@nowhere.com>");
        assertTrue("addr should be testaddr "+addr.getUser(), 
                    addr.getUser().equals("testaddr"));
    }

    public void testGetDomain() {
        MailAddress addr = MailAddress.parseSMTPStyle("<testaddr@nowhere.com>");
        assertTrue("addr should be testaddr "+addr.getDomain(), 
                    addr.getDomain().equals("nowhere.com"));
    }

}
