/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;

import org.jboss.mail.message.MailAddress;
import org.jboss.mail.smtp.SMTPConstants;
import org.jboss.mail.smtp.SMTPRequest;
import org.jboss.mail.smtp.SMTPResponse;

/**
 * Test the CmdRCPT handler
 * @author Andrew C. Oliver
 */
public class TestCmdRCPT extends HandlerBase {

    public TestCmdRCPT(String name){
        super(name);
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestCmdRCPT.class);
    }

    /*
     * @see HandlerBase#setUp()
     */
    protected void setUp() throws Exception {
        super.setUp();
        handler = new CmdRCPT();
    }

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.HandlerBase#testHandleRequest()
     */
    public void testHandleRequest() throws Exception {
        ByteArrayInputStream in = new ByteArrayInputStream("".getBytes());
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        SMTPRequest request =
            createRequest("RCPT", in, new String[] { "TO:<test@localhost>" });

        //set sender (this WOULD happen in the MAIL command)
        MailAddress address = MailAddress.parseSMTPStyle("<test@localhost>");
        super.setProtocolState(SMTPConstants.SENDER, address);

        SMTPResponse response = this.handleRequest(out, request);
        String res = out.toString();
        System.out.println("res was " + res);

        assertTrue("response can't be null", response != null);
        String expected = "250 Recipient <test@localhost> OK";
        assertTrue(
            "expected: " + expected + ", got=" + res.toString().trim(),
            expected.equals(res.toString().trim()));
    }

}
