/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.smtp.handlers;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;

import javax.management.Attribute;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.naming.InitialContext;

import org.jboss.mail.MBeanServerUtil;
import org.jboss.mail.Server;
import org.jboss.mail.util.JBMailJMXUtil;
import org.jboss.security.plugins.JaasSecurityDomain;

/**
 * Test the CmdSTARTTLS handler
 * @author Michael Krause
 * "author Kabir Khan
 */
public class TestCmdSTARTTLS extends HandlerBase {
     MBeanServerUtil mbsu;
     Server server = null;
	 
    public TestCmdSTARTTLS(String name){
        super(name);
     }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(TestCmdSTARTTLS.class);
    }

    protected void tearDown() throws Exception{ 
    }

    /*
     * @see HandlerBase#setUp()
     */
    protected void setUp() throws Exception {
    	//Not really necessary in this case
        super.setUp();
        
        MBeanServerUtil.configureMBeanServerFactory();
        MBeanServerUtil.setupSysout();
        
        //server = new Server(9000, 5, "localhost", "SMTP", 30000, 120000);
        server = new Server();
        //server.setProtocolFactoryName("class:org.jboss.mail.ProtocolFactory");
        
        MBeanServer mbserver = JBMailJMXUtil.locateJBoss();
        
        //Register the SMTPServer
        ObjectName servername = new ObjectName("jboss.mail:type=MailServices,name=SMTP");
        mbserver.registerMBean(server, servername);
        
		//Register and create the security domain
		JaasSecurityDomain securityDomain = new JaasSecurityDomain("Mail+SSL");
		securityDomain.setKeyStoreURL("mailservice.keystore");
		securityDomain.setKeyStorePass("rmi+ssl");
		securityDomain.create();
		ObjectName securityname = new ObjectName("jboss.security:service=JaasSecurityDomain,domain=Mail+SSL");
		mbserver.registerMBean(securityDomain, securityname);
		
		//Put the security doimain into the protocol and start it
		ObjectName protocolname = new ObjectName("jboss.mail:type=MailServices,name=SMTPProtocol");
		mbserver.setAttribute(protocolname, new Attribute("SecurityDomain", "java:/jaas/Mail+SSL"));
		mbserver.invoke(protocolname, "start",new Object[0], new String[0]);
		

        server.setProtocol("jboss.mail:type=MailServices,name=SMTPProtocol");
        server.setPoolSize(100);
        server.setLife(120000);
        server.setTimeout(30000);
        server.setAddress("localhost");
        server.setPort(9000);
        server.setBacklog(5);
        MBeanServerUtil.startServerService(server);
        Thread.sleep(500);
        
        handler = new CmdSTARTTLS();
        
    }
    

    /* (non-Javadoc)
     * @see org.jboss.mail.smtp.handlers.HandlerBase#testHandleRequest()
     */
    public void testHandleRequest() throws Exception {
    	//Send HELO
    	String request = "EHLO jboss.org" + newline; 
        Socket sock = new Socket("localhost",9000);
        OutputStream stream = sock.getOutputStream();
        stream.write(request.getBytes());
    	InputStream in = sock.getInputStream();
        BufferedReader rdr = new BufferedReader(new InputStreamReader(new BufferedInputStream(in)));
        
		//Send Get response to HELO
		String res = rdr.readLine();//220 localhost SMTP Server (JBMAIL SMTP Server version 0.1) ready...
		res = rdr.readLine();//250-localhost Hello jboss.org (127.0.0.1 (127.0.0.1))
		res = rdr.readLine();//250 AUTH LOGIN PLAIN
		
		request = "STARTTLS" + newline;
		stream.write(request.getBytes());
		res = rdr.readLine();
		System.out.println("res was:" + res);

		InitialContext ctx = new InitialContext();

      	String expected = "220 Ready to start TLS";
        assertTrue(
            "expected: " + expected + ", got=" + res.toString().trim(),
            expected.equals(res.toString().trim()));
    }

}
