/*
 * JBoss, the OpenSource J2EE webOS
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.mail.maillistener;

import javax.management.ObjectName;

import org.jboss.mail.ServiceProxy;
import org.jboss.mail.ServiceProxyFactory;
import org.jboss.mail.message.MailAddress;
import org.jboss.mail.message.Message;
import org.jboss.system.ServiceMBean;
import org.w3c.dom.Element;

/**
 * Proof of concept implementation which deals with posting your mails to JMS..  See
 * JMSMailListener for more info.
 *
 * @author Andrew C. Oliver
 */
public interface JMSMailListenerMBean extends ServiceMBean, ServiceProxyFactory {
    /**
     * set the type of destination to queue or topic
     * @param string "queue" or "topic"
     */
    void setDestinationType(String string);
    /**
     * @param string queue or topic to post to (for domains in the onServerDomainsList)
     */
    void setOnServerPostDestination(String string);
    /**
     * @return "queue" or "topic" whether we post to a queue or a topic
     */
    String getDestinationType();
    /**
     * @param string queue or topic to post to for domains not in the onServerDomainsList
     */
    void setOffServerPostDestination(String string);
    /**
     * @return String name of the queue or topic we're posting to for domains in the onServerDomainsList
     */
    String getOnServerPostDestination();
    /**
     * @return STring name of the queue or topic we're posting to for domains NOT in the onServerDomainsList
     */
    String getOffServerPostDestination();
    /**
     * @param domainGroup domain group reference
     */
    void setDomainGroup(ObjectName domainGroup);
    /**
     * @return domainGroup domain group reference
     */
    ObjectName getDomainGroup();

    /**
     * @param ma the address we will send to
     * @return either the onServer or offServer queue/topic name depending on whether the mail is local or not
     */
    String getDestinationForMailAddress(MailAddress ma);

    /*
     * non javadoc: see ServiceProxyFactory interface
     */
    ServiceProxy produceServiceProxy();
    
    /** Sets the times between retries for local delivery
     * @param Element representing xml of form: <BR>
     * &lt;localRetryTimes&gt;&lt;retryTime&gt;30&lt;/retryTime&gt;&lt;retryTime&gt;120&lt;/retryTime&gt;&lt;/localRetryTimes&gt;<BR>
     * Means maximum two retries will happen, the first will wait 30 seconds, and the second 120 seconds
     * 
     */
    void setLocalRetryTimes(Element localRetryTimes);

    /** Gets the times between retries for local delivery
     * @param Element representing xml of form: <BR>
     */ 
    Element getLocalRetryTimes();

    /** Sets the times between retries for remote delivery
     * @param Element representing xml of form: <BR>
     * &lt;remoteRetryTimes&gt;&lt;retryTime&gt;30&lt;/retryTime&gt;&lt;retryTime&gt;120&lt;/retryTime&gt;&lt;/remoteRetryTimes&gt;<BR>
     * Means maximum two retries will happen, the first will wait 30 seconds, and the second 120 seconds
     * 
     */
    void setRemoteRetryTimes(Element remoteRetryTimes);

    /** Gets the times between retries for remote delivery
     * @param Element representing xml of form: <BR>
     */ 
    Element getRemoteRetryTimes();


    /**
     * actually send the message.  This is used by the JMXMailListenerProxy to ACTUALLY send the mail
     * @param msg mail to send (we'll have to cast it to Mail)
     * @return the message containing what could not be handled by this listener, null if 
     * the whole message was consumed
     */
    Message send(Message msg);

}


